@@ -177,7 +177,6 @@ static int ipa_ldap_bind(const char *ldap_uri, krb5_principal bind_princ,
                          const char *mech, const char *ca_cert_file,
                          LDAP **_ld)
 {
-    char *msg = NULL;
     struct berval bv;
     LDAP *ld;
     int ret;
@@ -205,7 +204,7 @@ static int ipa_ldap_bind(const char *ldap_uri, krb5_principal bind_princ,
         ret = ldap_sasl_bind_s(ld, bind_dn, LDAP_SASL_SIMPLE,
                                &bv, NULL, NULL, NULL);
         if (ret != LDAP_SUCCESS) {
-            fprintf(stderr, _("Simple bind failed\n"));
+            ipa_ldap_error(ld, ret, _("Simple bind failed\n"));
             goto done;
         }
     } else {
@@ -219,11 +218,7 @@ static int ipa_ldap_bind(const char *ldap_uri, krb5_principal bind_princ,
         }
 
         if (ret != LDAP_SUCCESS) {
-#ifdef LDAP_OPT_DIAGNOSTIC_MESSAGE
-            ldap_get_option(ld, LDAP_OPT_DIAGNOSTIC_MESSAGE, (void*)&msg);
-#endif
-            fprintf(stderr, "SASL Bind failed %s (%d) %s!\n",
-                            ldap_err2string(ret), ret, msg ? msg : "");
+            ipa_ldap_error(ld, ret, _("SASL Bind failed\n"));
             goto done;
         }
     }
@@ -512,6 +507,7 @@ static int ldap_get_keytab(krb5_context krbctx, bool generate, char *password,
     if (enctypes) {
         ret = ipa_string_to_enctypes(enctypes, &es, &num_es, err_msg);
         if (ret || num_es == 0) {
+            free(es);
             return LDAP_OPERATIONS_ERROR;
         }
     }
@@ -559,7 +555,16 @@ done:
     return ret;
 }
 
-static char *ask_password(krb5_context krbctx)
+/* Prompt for either a password.
+ * This can be either asking for a new or existing password.
+ *
+ * To set a new password provide values for both prompt1 and prompt2 and
+ * set match=true to enforce that the two entered passwords match.
+ *
+ * To prompt for an existing password provide prompt1 and set match=false.
+ */
+static char *ask_password(krb5_context krbctx, char *prompt1, char *prompt2,
+                          bool match)
 {
     krb5_prompt ap_prompts[2];
     krb5_data k5d_pw0;
@@ -567,24 +572,27 @@ static char *ask_password(krb5_context krbctx)
     char pw0[256];
     char pw1[256];
     char *password;
+    int num_prompts = match ? 2:1;
 
     k5d_pw0.length = sizeof(pw0);
     k5d_pw0.data = pw0;
-    ap_prompts[0].prompt = _("New Principal Password");
+    ap_prompts[0].prompt = prompt1;
     ap_prompts[0].hidden = 1;
     ap_prompts[0].reply = &k5d_pw0;
 
-    k5d_pw1.length = sizeof(pw1);
-    k5d_pw1.data = pw1;
-    ap_prompts[1].prompt = _("Verify Principal Password");
-    ap_prompts[1].hidden = 1;
-    ap_prompts[1].reply = &k5d_pw1;
+    if (match) {
+        k5d_pw1.length = sizeof(pw1);
+        k5d_pw1.data = pw1;
+        ap_prompts[1].prompt = prompt2;
+        ap_prompts[1].hidden = 1;
+        ap_prompts[1].reply = &k5d_pw1;
+    }
 
     krb5_prompter_posix(krbctx, NULL,
                 NULL, NULL,
-                2, ap_prompts);
+                num_prompts, ap_prompts);
 
-    if (strcmp(pw0, pw1)) {
+    if (match && (strcmp(pw0, pw1))) {
         fprintf(stderr, _("Passwords do not match!"));
         return NULL;
     }
@@ -672,6 +680,56 @@ int read_ipa_config(struct ipa_config **ipacfg)
     return 0;
 }
 
+static int resolve_ktname(const char *keytab, char **ktname, char **err_msg)
+{
+	char keytab_resolved[PATH_MAX + 1];
+	struct stat st;
+	struct stat lst;
+	int ret;
+
+	*err_msg = NULL;
+
+    /* Resolve keytab symlink to support dangling symlinks, see
+     * https://pagure.io/freeipa/issue/4607. To prevent symlink attacks,
+     * the symlink is only resolved owned by the current user or by
+     * root. For simplicity, only one level if indirection is resolved.
+     */
+    if ((stat(keytab, &st) == -1) &&
+            (errno == ENOENT) &&
+            (lstat(keytab, &lst) == 0) &&
+            (S_ISLNK(lst.st_mode))) {
+        /* keytab is a dangling symlink. */
+        if (((lst.st_uid == 0) && (lst.st_gid == 0)) ||
+                ((lst.st_uid == geteuid()) && (lst.st_gid == getegid()))) {
+            /* Either root or current user owns symlink, resolve symlink and
+             * return the resolved symlink. */
+            ret = readlink(keytab, keytab_resolved, PATH_MAX + 1);
+            if ((ret == -1) || (ret > PATH_MAX)) {
+                *err_msg = _("Failed to resolve symlink to keytab.\n");
+                return ENOENT;
+            }
+            keytab_resolved[ret] = '\0';
+            ret = asprintf(ktname, "WRFILE:%s", keytab_resolved);
+            if (ret == -1) {
+                *err_msg = strerror(errno);
+                return ENOMEM;
+            }
+            return 0;
+        } else {
+            *err_msg = _("keytab is a dangling symlink and owned by another "
+                         "user.\n");
+            return EINVAL;
+        }
+    } else {
+        ret = asprintf(ktname, "WRFILE:%s", keytab);
+        if (ret == -1) {
+            *err_msg = strerror(errno);
+            return ENOMEM;
+        }
+        return 0;
+    }
+}
+
 int main(int argc, const char *argv[])
 {
 	static const char *server = NULL;
@@ -685,6 +743,7 @@ int main(int argc, const char *argv[])
 	static const char *ca_cert_file = NULL;
 	int quiet = 0;
 	int askpass = 0;
+	int askbindpw = 0;
 	int permitted_enctypes = 0;
 	int retrieve = 0;
         struct poptOption options[] = {
@@ -712,6 +771,8 @@ int main(int argc, const char *argv[])
               _("LDAP DN"), _("DN to bind as if not using kerberos") },
 	    { "bindpw", 'w', POPT_ARG_STRING, &bindpw, 0,
               _("LDAP password"), _("password to use if not using kerberos") },
+	    { NULL, 'W', POPT_ARG_NONE, &askbindpw, 0,
+              _("Prompt for LDAP password"), NULL },
 	    { "cacert", 0, POPT_ARG_STRING, &ca_cert_file, 0,
               _("Path to the IPA CA certificate"), _("IPA CA certificate")},
 	    { "ldapuri", 'H', POPT_ARG_STRING, &ldap_uri, 0,
@@ -783,9 +844,24 @@ int main(int argc, const char *argv[])
 		exit(2);
 	}
 
+    if (askbindpw && bindpw != NULL) {
+		fprintf(stderr, _("Bind password already provided (-w).\n"));
+		if (!quiet) {
+			poptPrintUsage(pc, stderr, 0);
+		}
+		exit(2);
+    }
+
+    if (askbindpw) {
+		bindpw = ask_password(krbctx, _("Enter LDAP password"), NULL, false);
+		if (!bindpw) {
+			exit(2);
+		}
+    }
+
 	if (NULL!=binddn && NULL==bindpw) {
 		fprintf(stderr,
-                        _("Bind password required when using a bind DN.\n"));
+                        _("Bind password required when using a bind DN (-w or -W).\n"));
 		if (!quiet)
 			poptPrintUsage(pc, stderr, 0);
 		exit(10);
@@ -849,7 +925,8 @@ int main(int argc, const char *argv[])
     }
 
         if (askpass) {
-		password = ask_password(krbctx);
+		password = ask_password(krbctx, _("New Principal Password"),
+               	                _("Verify Principal Password"), true);
 		if (!password) {
 			exit(2);
 		}
@@ -860,11 +937,6 @@ int main(int argc, const char *argv[])
 		}
 	}
 
-	ret = asprintf(&ktname, "WRFILE:%s", keytab);
-	if (ret == -1) {
-		exit(3);
-	}
-
 	krberr = krb5_parse_name(krbctx, principal, &sprinc);
 	if (krberr) {
 		fprintf(stderr, _("Invalid Service Principal Name\n"));
@@ -889,6 +961,12 @@ int main(int argc, const char *argv[])
 		}
 	}
 
+	ret = resolve_ktname(keytab, &ktname, &err_msg);
+	if (krberr) {
+		fprintf(stderr, "%s", err_msg);
+		exit(ret);
+	}
+
 	krberr = krb5_kt_resolve(krbctx, ktname, &kt);
 	if (krberr) {
 		fprintf(stderr, _("Failed to open Keytab\n"));
