@@ -24,6 +24,7 @@
 #include <sys/utsname.h>
 
 #include "ipa_kdb.h"
+#include "ipa_krb5.h"
 
 #define IPADB_GLOBAL_CONFIG_CACHE_TIME 60
 
@@ -193,6 +194,8 @@ static const struct {
     { "password", IPADB_USER_AUTH_PASSWORD },
     { "radius", IPADB_USER_AUTH_RADIUS },
     { "otp", IPADB_USER_AUTH_OTP },
+    { "pkinit", IPADB_USER_AUTH_PKINIT },
+    { "hardened", IPADB_USER_AUTH_HARDENED },
     { }
 };
 
@@ -586,8 +589,9 @@ static krb5_error_code ipadb_init_module(krb5_context kcontext,
 
     ret = ipadb_get_connection(ipactx);
     if (ret != 0) {
-        /* not a fatal failure, as the LDAP server may be temporarily down */
-        /* TODO: spam syslog with this error */
+        /* Not a fatal failure, as the LDAP server may be temporarily down. */
+        krb5_klog_syslog(LOG_INFO,
+                         "Didn't connect to LDAP on startup: %d", ret);
     }
 
     kerr = krb5_db_set_context(kcontext, ipactx);
@@ -631,57 +635,11 @@ static krb5_error_code ipadb_get_age(krb5_context kcontext,
     return 0;
 }
 
-#if KRB5_KDB_DAL_MAJOR_VERSION == 5
-static void *ipadb_alloc(krb5_context context, void *ptr, size_t size)
-{
-    return realloc(ptr, size);
-}
-
-static void ipadb_free(krb5_context context, void *ptr)
-{
-    free(ptr);
-}
-#endif
-
 /* KDB Virtual Table */
 
 /* We explicitly want to keep different ABI tables below separate. */
 /* Do not merge them together. Older ABI does not need to be updated */
 
-#if KRB5_KDB_DAL_MAJOR_VERSION == 5
-kdb_vftabl kdb_function_table = {
-    .maj_ver = KRB5_KDB_DAL_MAJOR_VERSION,
-    .min_ver = 0,
-    .init_library = ipadb_init_library,
-    .fini_library = ipadb_fini_library,
-    .init_module = ipadb_init_module,
-    .fini_module = ipadb_fini_module,
-    .create = ipadb_create,
-    .get_age = ipadb_get_age,
-    .get_principal = ipadb_get_principal,
-    .free_principal = ipadb_free_principal,
-    .put_principal = ipadb_put_principal,
-    .delete_principal = ipadb_delete_principal,
-    .iterate = ipadb_iterate,
-    .create_policy = ipadb_create_pwd_policy,
-    .get_policy = ipadb_get_pwd_policy,
-    .put_policy = ipadb_put_pwd_policy,
-    .iter_policy = ipadb_iterate_pwd_policy,
-    .delete_policy = ipadb_delete_pwd_policy,
-    .free_policy = ipadb_free_pwd_policy,
-    .alloc = ipadb_alloc,
-    .free = ipadb_free,
-    .fetch_master_key = ipadb_fetch_master_key,
-    .store_master_key_list = ipadb_store_master_key_list,
-    .change_pwd = ipadb_change_pwd,
-    .sign_authdata = ipadb_sign_authdata,
-    .check_transited_realms = ipadb_check_transited_realms,
-    .check_policy_as = ipadb_check_policy_as,
-    .audit_as_req = ipadb_audit_as_req,
-    .check_allowed_to_delegate = ipadb_check_allowed_to_delegate
-};
-#endif
-
 #if (KRB5_KDB_DAL_MAJOR_VERSION == 6) && !defined(HAVE_KDB_FREEPRINCIPAL_EDATA)
 kdb_vftabl kdb_function_table = {
     .maj_ver = KRB5_KDB_DAL_MAJOR_VERSION,
@@ -747,8 +705,72 @@ kdb_vftabl kdb_function_table = {
 };
 #endif
 
-#if (KRB5_KDB_DAL_MAJOR_VERSION != 5) && \
-    (KRB5_KDB_DAL_MAJOR_VERSION != 6) && \
-    (KRB5_KDB_DAL_MAJOR_VERSION != 7)
+#if (KRB5_KDB_DAL_MAJOR_VERSION == 8)
+/* Version 8 adds several arguments here.  However, if we want to actually use
+ * them in mspac, we really ought to drop support for older DAL versions. */
+static inline krb5_error_code
+stub_sign_authdata(krb5_context context, unsigned int flags,
+                   krb5_const_principal client_princ,
+                   krb5_const_principal server_princ, krb5_db_entry *client,
+                   krb5_db_entry *server, krb5_db_entry *header_server,
+                   krb5_db_entry *local_tgt, krb5_keyblock *client_key,
+                   krb5_keyblock *server_key, krb5_keyblock *header_key,
+                   krb5_keyblock *local_tgt_key, krb5_keyblock *session_key,
+                   krb5_timestamp authtime, krb5_authdata **tgt_auth_data,
+                   void *ad_info, krb5_data ***auth_indicators,
+                   krb5_authdata ***signed_auth_data)
+{
+    krb5_db_entry *krbtgt = header_server ? header_server : local_tgt;
+    krb5_keyblock *krbtgt_key = header_key ? header_key : local_tgt_key;
+
+    if (flags & KRB5_KDB_FLAG_CONSTRAINED_DELEGATION) {
+        client = header_server;
+        krbtgt = local_tgt;
+        krbtgt_key = local_tgt_key;
+    }
+
+    return ipadb_sign_authdata(context, flags, client_princ, client, server,
+                               krbtgt, client_key, server_key, krbtgt_key,
+                               session_key, authtime, tgt_auth_data,
+                               signed_auth_data);
+}
+
+kdb_vftabl kdb_function_table = {
+    .maj_ver = KRB5_KDB_DAL_MAJOR_VERSION,
+    .min_ver = 0,
+    .init_library = ipadb_init_library,
+    .fini_library = ipadb_fini_library,
+    .init_module = ipadb_init_module,
+    .fini_module = ipadb_fini_module,
+    .create = ipadb_create,
+    .get_age = ipadb_get_age,
+    .get_principal = ipadb_get_principal,
+    .put_principal = ipadb_put_principal,
+    .delete_principal = ipadb_delete_principal,
+    .iterate = ipadb_iterate,
+    .create_policy = ipadb_create_pwd_policy,
+    .get_policy = ipadb_get_pwd_policy,
+    .put_policy = ipadb_put_pwd_policy,
+    .iter_policy = ipadb_iterate_pwd_policy,
+    .delete_policy = ipadb_delete_pwd_policy,
+    .fetch_master_key = ipadb_fetch_master_key,
+    .store_master_key_list = ipadb_store_master_key_list,
+    .change_pwd = ipadb_change_pwd,
+    .sign_authdata = stub_sign_authdata,
+    .check_transited_realms = ipadb_check_transited_realms,
+    .check_policy_as = ipadb_check_policy_as,
+    .audit_as_req = ipadb_audit_as_req,
+    .check_allowed_to_delegate = ipadb_check_allowed_to_delegate,
+    .free_principal_e_data = ipadb_free_principal_e_data,
+    .get_s4u_x509_principal = NULL,
+    .allowed_to_delegate_from = NULL,
+    .get_authdata_info = NULL,
+    .free_authdata_info = NULL,
+};
+#endif
+
+#if (KRB5_KDB_DAL_MAJOR_VERSION != 6) && \
+    (KRB5_KDB_DAL_MAJOR_VERSION != 7) && \
+    (KRB5_KDB_DAL_MAJOR_VERSION != 8)
 #error unsupported DAL major version
 #endif
