@@ -268,7 +268,6 @@ return {
             facet_group: 'dnsrecord',
             nested_entity : 'dnsrecord',
             name: 'records',
-            deleter_dialog: IPA.dns.record_search_deleter_dialog,
             title: '@mo:dnszone.label_singular',
             label: '@mo:dnsrecord.label',
             tab_label: '@mo:dnsrecord.label',
@@ -291,6 +290,7 @@ return {
     ],
     standard_association_facets: true,
     adder_dialog: {
+        title: '@i18n:objects.dnszone.add',
         height: 300,
         sections: [
             {
@@ -328,6 +328,9 @@ return {
         policies: [
             IPA.add_dns_zone_name_policy
         ]
+    },
+    deleter_dialog: {
+        title: '@i18n:objects.dnszone.remove'
     }
 };};
 
@@ -434,6 +437,7 @@ return {
         }
     ],
     adder_dialog: {
+        title: '@i18n:objects.dnsforwardzone.add',
         sections: [
             {
                 name: 'name',
@@ -500,6 +504,9 @@ return {
         policies: [
             IPA.add_dns_zone_name_policy
         ]
+    },
+    deleter_dialog: {
+        title: '@i18n:objects.dnsforwardzone.remove'
     }
 };};
 
@@ -875,7 +882,9 @@ IPA.dns.record_search_facet = function(spec) {
 
     var that = IPA.nested_search_facet(spec);
 
-    that.get_records = function(pkeys, on_success, on_error) {
+    that.get_records = function(records, pkeys_list, on_success, on_error) {
+
+        var pkeys = pkeys_list.keys;
 
         var batch = rpc.batch_command({
             name: that.get_records_command_name(),
@@ -887,6 +896,7 @@ IPA.dns.record_search_facet = function(spec) {
 
         for (var i=0; i<pkeys.length; i++) {
             var pkey = pkeys[i];
+            var call_pkey = pkeys_list.get(pkey);
 
             var command = rpc.command({
                 entity: that.table.entity.name,
@@ -975,6 +985,16 @@ IPA.dns.record_search_deleter_dialog = function(spec) {
     return that;
 };
 
+/**
+ * This definition of records attributes is used for generating adder dialog.
+ *
+ * HOW TO EDIT ADDER DIALOG:
+ * In case you want to edit fields or widgets in adder dialog, you have to
+ * create object from attribute, then the attribute will be the name property
+ * of object. Then most of the widget or fields attributes have to be added in
+ * widget_opt (field_opt). Check the IPA.dns.record_prepare_editor_for_type
+ * method.
+ */
 IPA.dns.record_metadata = null;
 IPA.dns.get_record_metadata = function() {
 
@@ -1180,10 +1200,10 @@ IPA.dns.get_record_metadata = function() {
         {
             name: 'srvrecord',
             attributes: [
-                'srv_part_priority',
-                'srv_part_weight',
-                'srv_part_port',
-                'srv_part_target'
+               'srv_part_priority',
+               'srv_part_weight',
+               'srv_part_port',
+               'srv_part_target'
             ],
             adder_attributes: [],
             columns: ['srv_part_priority', 'srv_part_weight', 'srv_part_port',
@@ -1226,6 +1246,19 @@ IPA.dns.get_record_metadata = function() {
             ],
             adder_attributes: [],
             columns: ['txt_part_data']
+        },
+        {
+            name: 'urirecord',
+            attributes: [
+                'uri_part_priority',
+                'uri_part_weight',
+                'uri_part_target'
+            ],
+            adder_attributes: [],
+            columns: [
+                    'uri_part_priority', 'uri_part_weight',
+                    'uri_part_target'
+            ]
         }
     ];
 
@@ -1317,6 +1350,7 @@ return {
         }
     ],
     adder_dialog: {
+        title: '@i18n:objects.dnsrecord.add',
         $factory: IPA.dns.record_adder_dialog,
         fields: [
             {
@@ -1350,6 +1384,10 @@ return {
                 type_field: 'record_type'
             }
         ]
+    },
+    deleter_dialog: {
+        title: '@i18n:objects.dnsrecord.remove',
+        $factory: IPA.dns.record_search_deleter_dialog
     }
 };};
 
@@ -1509,14 +1547,15 @@ IPA.dns.record_prepare_editor_for_type = function(type, fields, widgets, update)
 
         //create editor widget
         var widget = {};
-        if (typeof attribute === 'string') {
-            widget.name = attribute;
-        } else {
-            widget.name = attribute.name;
-            set_defined(attribute.$type, widget, '$type');
-            set_defined(attribute.options, widget, 'options');
-            copy_obj(widget, attribute.widget_opt);
+        widget.name = attribute.name;
+        if (metadata) {
+            var doc = metadata.doc;
+            var label = metadata.label;
+            if (doc !== label) widget.tooltip = doc;
         }
+        set_defined(attribute.$type, widget, '$type');
+        set_defined(attribute.options, widget, 'options');
+        copy_obj(widget, attribute.widget_opt);
         section.widgets.push(widget);
     }
 };
@@ -1608,7 +1647,7 @@ IPA.dns_record_types = function() {
     //only supported
     var attrs = ['A', 'AAAA', 'A6', 'AFSDB', 'CERT', 'CNAME', 'DNAME',
                    'DS', 'DLV', 'KX', 'LOC', 'MX', 'NAPTR', 'NS',
-                   'PTR', 'SRV', 'SSHFP', 'TLSA', 'TXT'];
+                   'PTR', 'SRV', 'SSHFP', 'TLSA', 'TXT', 'URI'];
     var record_types = [];
     for (var i=0; i<attrs.length; i++) {
         var attr = attrs[i];
@@ -1922,7 +1961,7 @@ IPA.dns.record_type_table_widget = function(spec) {
 
         if (!selected_values.length) {
             var message = text.get('@i18n:dialogs.remove_empty');
-            alert(message);
+            window.alert(message);
             return;
         }
 
@@ -1970,15 +2009,12 @@ IPA.dns.record_type_table_widget = function(spec) {
 
     that.create_add_dialog = function() {
 
-        var title = text.get('@i18n:dialogs.add_title');
-        var label = that.entity.metadata.label_singular;
-
         var dialog_spec = {
             entity: that.entity,
             fields: [],
             widgets: [],
             retry: false,
-            title: title.replace('${entity}', label)
+            title: text.get('@i18n:objects.dnsrecord.add')
         };
 
         var dnstype = that.dnstype.toLowerCase();
