@@ -54,7 +54,7 @@ The tutorial examples all have this pattern:
         >>> class my_command(Command):
         ...     pass
         ...
-        >>> api.register(my_command)
+        >>> api.add_plugin(my_command)
         >>> api.finalize()
 
 In the tutorial we call `create_api()` to create an *example* instance
@@ -65,11 +65,13 @@ A real plugin will have this pattern:
 
     ::
 
-        from ipalib import Command, api
+        from ipalib import Command, Registry, api
 
+        register = Registry()
+
+        @register()
         class my_command(Command):
             pass
-        api.register(my_command)
 
 As seen above, also note that in a real plugin you will *not* call
 `plugable.API.finalize()`.  When in doubt, look at some of the built-in
@@ -104,7 +106,7 @@ thereof).  Here is our first example:
 >>> class my_command(Command): # Step 1, define class
 ...     """My example plugin."""
 ...
->>> api.register(my_command) # Step 2, register class
+>>> api.add_plugin(my_command) # Step 2, register class
 
 Notice that we are registering the ``my_command`` class itself, not an
 instance of ``my_command``.
@@ -138,7 +140,7 @@ implement a ``run()`` method, like this:
 ...         return dict(result='My run() method was called!')
 ...
 >>> api = create_api()
->>> api.register(my_command)
+>>> api.add_plugin(my_command)
 >>> api.finalize()
 >>> api.Command.my_command(version=u'2.47') # Call your command
 {'result': 'My run() method was called!'}
@@ -195,7 +197,7 @@ called:
 
 >>> api = create_api()
 >>> api.env.in_server = False # run() will dispatch to forward()
->>> api.register(my_command)
+>>> api.add_plugin(my_command)
 >>> api.finalize()
 >>> api.Command.my_command(version=u'2.47') # Call your command plugin
 {'result': 'forward(): in_server=False'}
@@ -205,7 +207,7 @@ On the other hand, if ``my_command`` is loaded in a *server* context,
 
 >>> api = create_api()
 >>> api.env.in_server = True # run() will dispatch to execute()
->>> api.register(my_command)
+>>> api.add_plugin(my_command)
 >>> api.finalize()
 >>> api.Command.my_command(version=u'2.47') # Call your command plugin
 {'result': 'execute(): in_server=True'}
@@ -261,7 +263,7 @@ Here is a simple example:
 ...         return 'Stuff got done.'
 ...
 >>> api = create_api()
->>> api.register(my_backend)
+>>> api.add_plugin(my_backend)
 >>> api.finalize()
 >>> api.Backend.my_backend.do_stuff()
 'Stuff got done.'
@@ -312,7 +314,7 @@ plugin:
 ...         """my_command.execute() calls this."""
 ...         return 'my_backend.do_stuff() indeed did do stuff!'
 ...
->>> api.register(my_backend)
+>>> api.add_plugin(my_backend)
 
 Second, we have our frontend plugin, the command:
 
@@ -323,7 +325,7 @@ Second, we have our frontend plugin, the command:
 ...         """Implemented against Backend.my_backend"""
 ...         return dict(result=self.Backend.my_backend.do_stuff())
 ...
->>> api.register(my_command)
+>>> api.add_plugin(my_command)
 
 Lastly, we call ``api.finalize()`` and see what happens when we call
 ``my_command()``:
@@ -349,7 +351,7 @@ example:
 ...     def forward(self, **options):
 ...         return dict(result='Just my_command.forward() getting called here.')
 ...
->>> api.register(my_command)
+>>> api.add_plugin(my_command)
 >>> api.finalize()
 
 Notice that the ``my_backend`` plugin has certainly not be registered:
@@ -391,9 +393,9 @@ several other commands in a single operation.  For example:
 ...     def execute(self, **options):
 ...         return dict(result='command_2.execute() called')
 ...
->>> api.register(meta_command)
->>> api.register(command_1)
->>> api.register(command_2)
+>>> api.add_plugin(meta_command)
+>>> api.add_plugin(command_1)
+>>> api.add_plugin(command_2)
 >>> api.finalize()
 >>> api.Command.meta_command(version=u'2.47')
 {'result': 'command_1.execute() called; command_2.execute() called.'}
@@ -428,7 +430,7 @@ For example:
 ...
 >>> api = create_api()
 >>> api.env.in_server = True
->>> api.register(nudge)
+>>> api.add_plugin(nudge)
 >>> api.finalize()
 >>> api.Command.nudge(u'Jason', version=u'2.47')
 {'result': u'Jason, go write more documentation!'}
@@ -525,7 +527,7 @@ the following:
 
 Also note that your ``execute()`` method should not contain any ``print``
 statements or otherwise cause any output on ``sys.stdout``.  Your command can
-(and should) produce log messages by using ``self.log`` (see below).
+(and should) produce log messages by using a module-level logger (see below).
 
 To learn more about XML-RPC (XML Remote Procedure Call), see:
 
@@ -616,7 +618,7 @@ For example, say we setup a command like this:
 ...
 >>> api = create_api()
 >>> api.bootstrap(in_server=True)  # We want to execute, not forward
->>> api.register(show_items)
+>>> api.add_plugin(show_items)
 >>> api.finalize()
 
 Normally when you invoke the ``ipa`` script, `cli.CLI.load_plugins()` will
@@ -668,18 +670,19 @@ See the `ipalib.cli.textui` plugin for a description of its methods.
 Logging from your plugin
 ------------------------
 
-After `plugable.Plugin.__init__()` is called, your plugin will have a
-``self.log`` attribute.  Plugins should only log through this attribute.
+Plugins should log through a module-level logger.
 For example:
 
+>>> import logging
+>>> logger = logging.getLogger(__name__)
 >>> class paint_house(Command):
 ...
 ...     takes_args = 'color'
 ...
 ...     def execute(self, color, **options):
-...         """Uses self.log.error()"""
+...         """Uses logger.error()"""
 ...         if color not in ('red', 'blue', 'green'):
-...             self.log.error("I don't have %s paint!", color) # Log error
+...             logger.error("I don't have %s paint!", color) # Log error
 ...             return
 ...         return 'I painted the house %s.' % color
 ...
@@ -689,7 +692,7 @@ Some basic knowledge of the Python ``logging`` module might be helpful. See:
     http://docs.python.org/library/logging.html
 
 The important thing to remember is that your plugin should not configure
-logging itself, but should instead simply use the ``self.log`` logger.
+logging itself, but should instead simply use the module-level logger.
 
 Also see the `plugable.API.bootstrap()` method for details on how the logging
 is configured.
@@ -719,7 +722,7 @@ Key            Example value                  Description
 =============  =============================  =======================
 bin            '/usr/bin'                     Dir. containing script
 dot_ipa        '/home/jderose/.ipa'           User config directory
-home           os.environ['HOME']             User home dir.
+home           os.path.expanduser('~')        User home dir.
 ipalib         '.../site-packages/ipalib'     Dir. of ipalib package
 mode           'unit_test'                    The mode ipalib is in
 script         sys.argv[0]                    Path of script
@@ -758,7 +761,7 @@ For example:
 ...
 >>> api = create_api()
 >>> api.bootstrap(in_server=True, message='Hello, world!')
->>> api.register(motd)
+>>> api.add_plugin(motd)
 >>> api.finalize()
 >>> api.Command.motd(version=u'2.47')
 {'result': u'Hello, world!'}
@@ -878,36 +881,81 @@ freeIPA.org:
     http://freeipa.org/page/Contribute
 
 '''
+from ipapython.version import VERSION as __version__
 
-import os
+def _enable_warnings(error=False):
+    """Enable additional warnings during development
+    """
+    import ctypes
+    import warnings
+
+    # get reference to Py_BytesWarningFlag from Python CAPI
+    byteswarnings = ctypes.c_int.in_dll(  # pylint: disable=no-member
+        ctypes.pythonapi, 'Py_BytesWarningFlag')
+
+    if byteswarnings.value >= 2:
+        # bytes warnings flag already set to error
+        return
+
+    # default warning mode for all modules: warn once per location
+    warnings.simplefilter('default', BytesWarning)
+    if error:
+        byteswarnings.value = 2
+        action = 'error'
+    else:
+        byteswarnings.value = 1
+        action = 'default'
+
+    module = '(ipa.*|__main__)'
+    warnings.filterwarnings(action, category=BytesWarning, module=module)
+    warnings.filterwarnings(action, category=DeprecationWarning,
+                            module=module)
+
+# call this as early as possible
+if 'git' in __version__:
+    _enable_warnings(False)
+
+# noqa: E402
 from ipalib import plugable
 from ipalib.backend import Backend
 from ipalib.frontend import Command, LocalOrRemote, Updater
 from ipalib.frontend import Object, Method
 from ipalib.crud import Create, Retrieve, Update, Delete, Search
-from ipalib.parameters import DefaultFrom, Bool, Flag, Int, Decimal, Bytes, Str, IA5Str, Password, DNParam, DeprecatedParam
+from ipalib.parameters import DefaultFrom, Bool, Flag, Int, Decimal, Bytes, Str, IA5Str, Password, DNParam
 from ipalib.parameters import (BytesEnum, StrEnum, IntEnum, AccessTime, File,
                         DateTime, DNSNameParam)
 from ipalib.errors import SkipPluginModule
 from ipalib.text import _, ngettext, GettextFactory, NGettextFactory
 
-version_info = (2, 0, 0, 'alpha', 0)
-if version_info[3] == 'final':
-    __version__ = '%d.%d.%d' % version_info[:3]
-else:
-    __version__ = '%d.%d.%d.%s.%d' % version_info
+Registry = plugable.Registry
 
 
 class API(plugable.API):
     bases = (Command, Object, Method, Backend, Updater)
 
     @property
-    def modules(self):
-        result = ('ipalib.plugins.*',)
+    def packages(self):
         if self.env.in_server:
-            result += ('ipaserver.plugins.*',)
+            # pylint: disable=import-error,ipa-forbidden-import
+            import ipaserver.plugins
+            # pylint: enable=import-error,ipa-forbidden-import
+            result = (
+                ipaserver.plugins,
+            )
+        else:
+            import ipaclient.remote_plugins
+            import ipaclient.plugins
+            result = (
+                ipaclient.remote_plugins.get_package(self),
+                ipaclient.plugins,
+            )
+
         if self.env.context in ('installer', 'updates'):
-            result += ('ipaserver.install.plugins.*',)
+            # pylint: disable=import-error,ipa-forbidden-import
+            import ipaserver.install.plugins
+            # pylint: enable=import-error,ipa-forbidden-import
+            result += (ipaserver.install.plugins,)
+
         return result
 
 
@@ -933,10 +981,3 @@ def create_api(mode='dummy'):
     return api
 
 api = create_api(mode=None)
-
-if os.environ.get('IPA_UNIT_TEST_MODE', None) == 'cli_test':
-    from ipalib.cli import cli_plugins
-    api.bootstrap(context='cli', in_server=False, in_tree=True)
-    for klass in cli_plugins:
-        api.add_plugin(klass)
-    api.finalize()
