@@ -41,7 +41,7 @@ PERMISSIONS = ["read", "write", "add", "delete", "search", "compare",
                "selfwrite", "proxy", "all"]
 
 
-class ACI(object):
+class ACI:
     """
     Holds the basic data for an ACI entry, as stored in the cn=accounts
     entry in LDAP.  Has methods to parse an ACI string and export to an
@@ -165,7 +165,7 @@ class ACI(object):
                 raise SyntaxError("invalid permission: '%s'" % p)
         if not self.name:
             raise SyntaxError("name must be set")
-        if not isinstance(self.name, six.string_types):
+        if not isinstance(self.name, str):
             raise SyntaxError("name must be a string")
         if not isinstance(self.target, dict) or len(self.target) == 0:
             raise SyntaxError("target must be a non-empty dictionary")
