@@ -33,12 +33,10 @@ from __future__ import absolute_import
 import os
 from os import path
 import sys
+from urllib.parse import urlparse, urlunparse
+from configparser import RawConfigParser, ParsingError
 
 import six
-# pylint: disable=import-error
-from six.moves.urllib.parse import urlparse, urlunparse
-from six.moves.configparser import RawConfigParser, ParsingError
-# pylint: enable=import-error
 
 from ipaplatform.tasks import tasks
 from ipapython.dn import DN
@@ -46,14 +44,15 @@ from ipalib.base import check_name
 from ipalib.constants import (
     CONFIG_SECTION,
     OVERRIDE_ERROR, SET_ERROR, DEL_ERROR,
-    TLS_VERSIONS
+    TLS_VERSIONS, TLS_VERSION_DEFAULT_MIN, TLS_VERSION_DEFAULT_MAX,
 )
 from ipalib import errors
 
 if six.PY3:
     unicode = str
 
-class Env(object):
+
+class Env:
     """
     Store and retrieve environment variables.
 
@@ -252,7 +251,7 @@ class Env(object):
             )
         # pylint: enable=no-member
         assert not hasattr(self, key)
-        if isinstance(value, six.string_types):
+        if isinstance(value, str):
             value = value.strip()
             if isinstance(value, bytes):
                 value = value.decode('utf-8')
@@ -451,7 +450,8 @@ class Env(object):
         self.site_packages = path.dirname(self.ipalib)
         self.script = path.abspath(sys.argv[0])
         self.bin = path.dirname(self.script)
-        self.home = os.environ.get('HOME', None)
+        home = os.path.expanduser('~')
+        self.home = home if not home.startswith('~') else None
         self.fips_mode = tasks.is_fips_enabled()
 
         # Merge in overrides:
@@ -632,20 +632,30 @@ class Env(object):
 
         # set the best known TLS version if min/max versions are not set
         if 'tls_version_min' not in self:
-            self.tls_version_min = TLS_VERSIONS[-1]
-        elif self.tls_version_min not in TLS_VERSIONS:
+            self.tls_version_min = TLS_VERSION_DEFAULT_MIN
+        if (
+                self.tls_version_min is not None and
+                self.tls_version_min not in TLS_VERSIONS
+        ):
             raise errors.EnvironmentError(
                 "Unknown TLS version '{ver}' set in tls_version_min."
                 .format(ver=self.tls_version_min))
 
         if 'tls_version_max' not in self:
-            self.tls_version_max = TLS_VERSIONS[-1]
-        elif self.tls_version_max not in TLS_VERSIONS:
+            self.tls_version_max = TLS_VERSION_DEFAULT_MAX
+        if (
+                self.tls_version_max is not None and
+                self.tls_version_max not in TLS_VERSIONS
+        ):
             raise errors.EnvironmentError(
                 "Unknown TLS version '{ver}' set in tls_version_max."
                 .format(ver=self.tls_version_max))
 
-        if self.tls_version_max < self.tls_version_min:
+        if (
+                self.tls_version_min is not None and
+                self.tls_version_max is not None and
+                self.tls_version_max < self.tls_version_min
+        ):
             raise errors.EnvironmentError(
                 "tls_version_min is set to a higher TLS version than "
                 "tls_version_max.")
