@@ -47,7 +47,7 @@ DBUS_CM_CA_IF = 'org.fedorahosted.certmonger.ca'
 DBUS_PROPERTY_IF = 'org.freedesktop.DBus.Properties'
 
 
-class _cm_dbus_object(object):
+class _cm_dbus_object:
     """
     Auxiliary class for convenient DBus object handling.
     """
@@ -179,6 +179,9 @@ def _get_requests(criteria=dict()):
         for criterion in criteria:
             if criterion == 'ca-name':
                 ca_path = request.obj_if.get_ca()
+                if ca_path is None:
+                    raise RuntimeError("certmonger CA '%s' is not defined" %
+                                       criteria.get('ca-name'))
                 ca = _cm_dbus_object(cm.bus, cm, ca_path, DBUS_CM_CA_IF,
                                      DBUS_CM_IF)
                 value = ca.obj_if.get_nickname()
@@ -326,7 +329,11 @@ def request_and_wait_for_cert(
 
     deadline = time.time() + resubmit_timeout
     while True:  # until success, timeout, or error
-        state = wait_for_request(req_id, api.env.replication_wait_timeout)
+        try:
+            state = wait_for_request(req_id, api.env.http_timeout)
+        except RuntimeError as e:
+            logger.debug("wait_for_request raised %s", e)
+            state = 'TIMEOUT'
         ca_error = get_request_value(req_id, 'ca-error')
         if state == 'MONITORING' and ca_error is None:
             # we got a winner, exiting
@@ -336,16 +343,19 @@ def request_and_wait_for_cert(
         logger.debug(
             "Cert request %s failed: %s (%s)", req_id, state, ca_error
         )
-        if state not in {'CA_REJECTED', 'CA_UNREACHABLE'}:
+        if state in {'CA_REJECTED', 'CA_UNREACHABLE'}:
             # probably unrecoverable error
             logger.debug("Giving up on cert request %s", req_id)
             break
-        elif not resubmit_timeout:
+        if not resubmit_timeout:
             # no resubmit
             break
-        elif time.time() > deadline:
-            logger.debug("Request %s reached resubmit dead line", req_id)
+        if time.time() > deadline:
+            logger.debug("Request %s reached resubmit deadline", req_id)
             break
+        if state == 'TIMEOUT':
+            logger.debug("%s not in final state, continue waiting", req_id)
+            time.sleep(10)
         else:
             # sleep and resubmit
             logger.debug("Sleep and resubmit cert request %s", req_id)
@@ -427,7 +437,8 @@ def request_cert(
 
 def start_tracking(
         certpath, ca='IPA', nickname=None, pin=None, pinfile=None,
-        pre_command=None, post_command=None, profile=None, storage="NSSDB"):
+        pre_command=None, post_command=None, profile=None, storage="NSSDB",
+        token_name=None):
     """
     Tell certmonger to track the given certificate in either a file or an NSS
     database. The certificate access can be protected by a password_file.
@@ -460,6 +471,8 @@ def start_tracking(
         NSS or OpenSSL backend to track the certificate in ``certpath``
     :param profile:
         Which certificate profile should be used.
+    :param token_name:
+        Hardware token name for HSM support
     :returns: certificate tracking nickname.
     """
     if storage == 'FILE':
@@ -500,6 +513,10 @@ def start_tracking(
         params['cert-postsave-command'] = post_command
     if profile:
         params['ca-profile'] = profile
+    if token_name not in {None, "internal"}:
+        # only pass token names for external tokens (e.g. HSM)
+        params['key-token'] = token_name
+        params['cert-token'] = token_name
 
     result = cm.obj_if.add_request(params)
     try:
@@ -590,6 +607,18 @@ def resubmit_request(
             update['template-is-ca'] = True
             update['template-ca-path-length'] = -1  # no path length
 
+        # TODO: certmonger assumes some hard-coded defaults like RSA 2048.
+        # Try to fetch values from current cert rather.
+        for key, convert in [('key-size', int), ('key-type', str)]:
+            try:
+                value = request.prop_if.Get(DBUS_CM_REQUEST_IF, key)
+            except dbus.DBusException:
+                continue
+            else:
+                if value:
+                    # convert dbus.Int64() to int, dbus.String() to str
+                    update[key] = convert(value)
+
         if len(update) > 0:
             request.obj_if.modify(update)
         request.obj_if.resubmit()
@@ -663,7 +692,7 @@ def modify_ca_helper(ca_name, helper):
         return old_helper
 
 
-def get_pin(token):
+def get_pin(token="internal"):
     """
     Dogtag stores its NSS pin in a file formatted as token:PIN.
 
