@@ -24,6 +24,7 @@ The classes in this module make heavy use of Python container emulation. If
 you are unfamiliar with this Python feature, see
 http://docs.python.org/ref/sequence-types.html
 """
+
 import logging
 import operator
 import re
@@ -46,7 +47,6 @@ from ipalib.base import ReadOnly, lock, islocked
 from ipalib.constants import DEFAULT_CONFIG
 from ipapython import ipa_log_manager, ipautil
 from ipapython.ipa_log_manager import (
-    log_mgr,
     LOGGING_FORMAT_FILE,
     LOGGING_FORMAT_STDERR)
 from ipapython.version import VERSION, API_VERSION, DEFAULT_PLUGINS
@@ -89,7 +89,7 @@ def find_modules_in_dir(src_dir):
         yield module
 
 
-class Registry(object):
+class Registry:
     """A decorator that makes plugins available to the API
 
     Usage::
@@ -144,7 +144,6 @@ class Plugin(ReadOnly):
         self.__finalize_called = False
         self.__finalized = False
         self.__finalize_lock = threading.RLock()
-        log_mgr.get_logger(self, True)
 
     @classmethod
     def __name_getter(cls):
@@ -175,7 +174,7 @@ class Plugin(ReadOnly):
     def __summary_getter(cls):
         doc = cls.doc
         if not _(doc).msg:
-            return u'<%s.%s>' % (cls.__module__, cls.__name__)
+            return '<%s.%s>' % (cls.__module__, cls.__name__)
         else:
             return unicode(doc).split('\n\n', 1)[0].strip()
 
@@ -239,7 +238,7 @@ class Plugin(ReadOnly):
             if not self.__finalized:
                 self.finalize()
 
-    class finalize_attr(object):
+    class finalize_attr:
         """
         Create a stub object for plugin attribute that isn't set until the
         finalization of the plugin initialization.
@@ -438,7 +437,6 @@ class API(ReadOnly):
         Initialize environment variables and logging.
         """
         self.__doing('bootstrap')
-        self.log = log_mgr.get_logger(self)
         self.env._bootstrap(**overrides)
         self.env._finalize_core(**dict(DEFAULT_CONFIG))
 
@@ -454,7 +452,7 @@ class API(ReadOnly):
         if root_logger.handlers or self.env.validate_api:
             return
 
-        if self.env.debug:
+        if self.env.debug:  # pylint: disable=using-constant-test
             level = logging.DEBUG
         else:
             level = logging.INFO
@@ -470,7 +468,7 @@ class API(ReadOnly):
             level = ipa_log_manager.convert_log_level(match.group(1))
 
             value = getattr(self.env, attr)
-            regexps = re.split('\s*,\s*', value)
+            regexps = re.split(r'\s*,\s*', value)
 
             # Add the regexp, it maps to the configured level
             for regexp in regexps:
@@ -478,7 +476,7 @@ class API(ReadOnly):
 
         # Add stderr handler:
         level = logging.INFO
-        if self.env.debug:
+        if self.env.debug:  # pylint: disable=using-constant-test
             level = logging.DEBUG
         else:
             if self.env.context == 'cli':
@@ -510,7 +508,7 @@ class API(ReadOnly):
                 return
 
         level = logging.INFO
-        if self.env.debug:
+        if self.env.debug:  # pylint: disable=using-constant-test
             level = logging.DEBUG
         try:
             handler = logging.FileHandler(self.env.log)
@@ -654,7 +652,8 @@ class API(ReadOnly):
                 logger.debug("skipping plugin module %s: %s", name, e.reason)
                 continue
             except Exception as e:
-                if self.env.startup_traceback:
+                tb = self.env.startup_traceback
+                if tb:  # pylint: disable=using-constant-test
                     logger.exception("could not load plugin module %s", name)
                 raise
 
