@@ -37,6 +37,7 @@ import sys
 import ssl
 import termios
 import fcntl
+import shutil
 import struct
 import subprocess
 
@@ -56,7 +57,8 @@ except ImportError:
 from ipalib import errors, messages
 from ipalib.constants import (
     DOMAIN_LEVEL_0,
-    TLS_VERSIONS, TLS_VERSION_MINIMAL
+    TLS_VERSIONS, TLS_VERSION_MINIMAL, TLS_VERSION_MAXIMAL,
+    TLS_VERSION_DEFAULT_MIN, TLS_VERSION_DEFAULT_MAX,
 )
 from ipalib.text import _
 from ipaplatform.constants import constants
@@ -86,7 +88,7 @@ def json_serialize(obj):
         return [json_serialize(o) for o in obj]
     if isinstance(obj, dict):
         return {k: json_serialize(v) for (k, v) in obj.items()}
-    if isinstance(obj, (bool, float, unicode, type(None), six.integer_types)):
+    if isinstance(obj, (int, bool, float, unicode, type(None))):
         return obj
     if isinstance(obj, str):
         return obj.decode('utf-8')
@@ -214,7 +216,7 @@ def check_writable_file(filename):
         raise errors.FileError(reason=str(e))
 
 def normalize_zonemgr(zonemgr):
-    if not zonemgr or not isinstance(zonemgr, six.string_types):
+    if not zonemgr or not isinstance(zonemgr, str):
         return zonemgr
     if '@' in zonemgr:
         # local-part needs to be normalized
@@ -246,6 +248,13 @@ def get_proper_tls_version_span(tls_version_min, tls_version_max):
         if lower than TLS_VERSION_MINIMAL
     :raises: ValueError
     """
+    if tls_version_min is None and tls_version_max is None:
+        # no defaults, use system's default TLS version range
+        return None
+    if tls_version_min is None:
+        tls_version_min = TLS_VERSION_MINIMAL
+    if tls_version_max is None:
+        tls_version_max = TLS_VERSION_MAXIMAL
     min_allowed_idx = TLS_VERSIONS.index(TLS_VERSION_MINIMAL)
 
     try:
@@ -281,8 +290,8 @@ def create_https_connection(
     cafile=None,
     client_certfile=None, client_keyfile=None,
     keyfile_passwd=None,
-    tls_version_min="tls1.1",
-    tls_version_max="tls1.2",
+    tls_version_min=TLS_VERSION_DEFAULT_MIN,
+    tls_version_max=TLS_VERSION_DEFAULT_MAX,
     **kwargs
 ):
     """
@@ -312,6 +321,7 @@ def create_https_connection(
         "tls1.0": ssl.OP_NO_TLSv1,
         "tls1.1": ssl.OP_NO_TLSv1_1,
         "tls1.2": ssl.OP_NO_TLSv1_2,
+        "tls1.3": getattr(ssl, "OP_NO_TLSv1_3", 0),
     }
     # pylint: enable=no-member
 
@@ -323,9 +333,6 @@ def create_https_connection(
         raise RuntimeError("cafile \'{file}\' doesn't exist or is unreadable".
                            format(file=cafile))
 
-    # remove the slice of negating protocol options according to options
-    tls_span = get_proper_tls_version_span(tls_version_min, tls_version_max)
-
     # official Python documentation states that the best option to get
     # TLSv1 and later is to setup SSLContext with PROTOCOL_SSLv23
     # and then negate the insecure SSLv2 and SSLv3
@@ -336,20 +343,28 @@ def create_https_connection(
         ssl.OP_SINGLE_ECDH_USE
     )
 
-    # high ciphers without RC4, MD5, TripleDES, pre-shared key and secure
-    # remote password. Uses system crypto policies on some platforms.
-    ctx.set_ciphers(constants.TLS_HIGH_CIPHERS)
+    if constants.TLS_HIGH_CIPHERS is not None:
+        # configure ciphers, uses system crypto policies on RH platforms.
+        ctx.set_ciphers(constants.TLS_HIGH_CIPHERS)
+
+    # remove the slice of negating protocol options according to options
+    tls_span = get_proper_tls_version_span(tls_version_min, tls_version_max)
 
     # pylint: enable=no-member
     # set up the correct TLS version flags for the SSL context
-    for version in TLS_VERSIONS:
-        if version in tls_span:
-            # make sure the required TLS versions are available if Python
-            # decides to modify the default TLS flags
-            ctx.options &= ~tls_cutoff_map[version]
-        else:
-            # disable all TLS versions not in tls_span
-            ctx.options |= tls_cutoff_map[version]
+    if tls_span is not None:
+        for version in TLS_VERSIONS:
+            if version in tls_span:
+                # make sure the required TLS versions are available if Python
+                # decides to modify the default TLS flags
+                ctx.options &= ~tls_cutoff_map[version]
+            else:
+                # disable all TLS versions not in tls_span
+                ctx.options |= tls_cutoff_map[version]
+
+    # Enable TLS 1.3 post-handshake auth
+    if getattr(ctx, "post_handshake_auth", None) is not None:
+        ctx.post_handshake_auth = True
 
     ctx.verify_mode = ssl.CERT_REQUIRED
     ctx.check_hostname = True
@@ -433,14 +448,28 @@ def validate_zonemgr_str(zonemgr):
     zonemgr = DNSName(zonemgr)
     return validate_zonemgr(zonemgr)
 
-def validate_hostname(hostname, check_fqdn=True, allow_underscore=False, allow_slash=False):
+
+def validate_hostname(hostname, check_fqdn=True, allow_underscore=False,
+                      allow_slash=False, maxlen=255):
     """ See RFC 952, 1123
 
+    Length limit of 64 imposed by MAXHOSTNAMELEN on Linux.
+
+    DNS and other operating systems has a max length of 255. Default to
+    the theoretical max unless explicitly told to limit. The cases
+    where a limit would be set might include:
+     * *-install --hostname
+     * ipa host-add
+
+    The *-install commands by definition are executed on Linux hosts so
+    the maximum length needs to be limited.
+
     :param hostname Checked value
     :param check_fqdn Check if hostname is fully qualified
     """
-    if len(hostname) > 255:
-        raise ValueError(_('cannot be longer that 255 characters'))
+    if len(hostname) > maxlen:
+        raise ValueError(_('cannot be longer that {} characters'.format(
+                         maxlen)))
 
     if hostname.endswith('.'):
         hostname = hostname[:-1]
@@ -763,8 +792,8 @@ def _resolve_record(owner, rtype, nameserver_ip=None, edns0=False,
     :param flag_cd: requires dnssec=True, adds flag CD
     :raise DNSException: if error occurs
     """
-    assert isinstance(nameserver_ip, six.string_types) or nameserver_ip is None
-    assert isinstance(rtype, six.string_types)
+    assert isinstance(nameserver_ip, str) or nameserver_ip is None
+    assert isinstance(rtype, str)
 
     res = dns.resolver.Resolver()
     if nameserver_ip:
@@ -994,7 +1023,7 @@ def has_managed_topology(api):
     return domainlevel > DOMAIN_LEVEL_0
 
 
-class classproperty(object):
+class classproperty:
     __slots__ = ('__doc__', 'fget')
 
     def __init__(self, fget=None, doc=None):
@@ -1028,9 +1057,16 @@ def normalize_hostname(hostname):
     return hostname
 
 
-def hostname_validator(ugettext, value):
+def hostname_validator(ugettext, value, maxlen=255):
+    """Validator used by plugins to ensure hostname compliance.
+
+       In Linux the maximum hostname length is 64. In DNS and
+       other operaring systems (Solaris) it is 255. If not explicitly
+       checking a Linux hostname (e.g. the server) use the DNS
+       default.
+    """
     try:
-        validate_hostname(value)
+        validate_hostname(value, maxlen=maxlen)
     except ValueError as e:
         return _('invalid domain-name: %s') % unicode(e)
 
@@ -1122,17 +1158,37 @@ def ensure_krbcanonicalname_set(ldap, entry_attrs):
     entry_attrs.update(old_entry)
 
 
-def check_client_configuration():
+def check_client_configuration(env=None):
     """
     Check if IPA client is configured on the system.
 
     Hardcode return code to avoid recursive imports
     """
-    if (not os.path.isfile(paths.IPA_DEFAULT_CONF) or
-            not os.path.isdir(paths.IPA_CLIENT_SYSRESTORE) or
-            not os.listdir(paths.IPA_CLIENT_SYSRESTORE)):
-        raise ScriptError('IPA client is not configured on this system',
-                          2)  # CLIENT_NOT_CONFIGURED
+    CLIENT_NOT_CONFIGURED = 2
+    if env is not None and env.confdir != paths.ETC_IPA:
+        # custom IPA conf dir, check for custom conf_default
+        if os.path.isfile(env.conf_default):
+            return True
+        else:
+            raise ScriptError(
+                f'IPA client is not configured on this system (confdir '
+                f'{env.confdir} is missing {env.conf_default})',
+                CLIENT_NOT_CONFIGURED
+            )
+    elif (
+            os.path.isfile(paths.IPA_DEFAULT_CONF) and
+            os.path.isfile(
+                os.path.join(paths.IPA_CLIENT_SYSRESTORE, 'sysrestore.state')
+            )
+    ):
+        # standard installation, check for config and client sysrestore state
+        return True
+    else:
+        # client not configured
+        raise ScriptError(
+            'IPA client is not configured on this system',
+            CLIENT_NOT_CONFIGURED
+        )
 
 
 def check_principal_realm_in_trust_namespace(api_instance, *keys):
@@ -1203,17 +1259,27 @@ def get_terminal_height(fd=1):
         return os.environ.get("LINES", 25)
 
 
-def open_in_pager(data):
+def get_pager():
+    """ Get path to a pager
+
+    :return: path to the file if it exists otherwise None
+    :rtype: str or None
+    """
+    pager = os.environ.get('PAGER', 'less')
+    return shutil.which(pager)
+
+
+def open_in_pager(data, pager):
     """
     Open text data in pager
 
     Args:
         data (bytes): data to view in pager
+        pager (str): path to the pager
 
     Returns:
         None
     """
-    pager = os.environ.get("PAGER", "less")
     pager_process = subprocess.Popen([pager], stdin=subprocess.PIPE)
 
     try:
