@@ -24,21 +24,12 @@ Contains Fedora-specific service class implementations.
 
 from __future__ import absolute_import
 
-from ipaplatform.osinfo import osinfo
 from ipaplatform.redhat import services as redhat_services
 
 # Mappings from service names as FreeIPA code references to these services
 # to their actual systemd service names
 fedora_system_units = redhat_services.redhat_system_units.copy()
 
-# Fedora 28 and earlier have fedora-domainname.service. Starting from
-# Fedora 29, the service is called nis-domainname.service as defined in
-# ipaplatform.redhat.services.
-HAS_FEDORA_DOMAINNAME_SERVICE = int(osinfo.version_id) <= 28
-
-if HAS_FEDORA_DOMAINNAME_SERVICE:
-    fedora_system_units['domainname'] = 'fedora-domainname.service'
-
 
 # Service classes that implement Fedora-specific behaviour
 
@@ -50,8 +41,6 @@ class FedoraService(redhat_services.RedHatService):
 # of specified name
 
 def fedora_service_class_factory(name, api=None):
-    if HAS_FEDORA_DOMAINNAME_SERVICE and name == 'domainname':
-        return FedoraService(name, api)
     return redhat_services.redhat_service_class_factory(name, api)
 
 
