@@ -21,13 +21,11 @@ import collections
 import gzip
 import io
 import logging
+from urllib.parse import urlencode
 import xml.dom.minidom
 import zlib
 
 import six
-# pylint: disable=import-error
-from six.moves.urllib.parse import urlencode
-# pylint: enable=import-error
 
 # pylint: disable=ipa-forbidden-import
 from ipalib import api, errors
@@ -229,7 +227,7 @@ def _httplib_request(
 
     try:
         conn = connection_factory(host, port, **connection_options)
-        conn.request(method, uri, body=request_body, headers=headers)
+        conn.request(method, path, body=request_body, headers=headers)
         res = conn.getresponse()
 
         http_status = res.status
