@@ -63,7 +63,6 @@ import pysss
 import six
 from ipaplatform.paths import paths
 
-from ldap.filter import escape_filter_chars
 from time import sleep
 
 try:
@@ -409,7 +408,12 @@ class DomainValidator:
         if object_name in result and \
            (pysss_nss_idmap.SID_KEY in result[object_name]):
             object_sid = result[object_name][pysss_nss_idmap.SID_KEY]
-            return object_sid
+            if self.is_trusted_sid_valid(object_sid):
+                return object_sid
+            else:
+                raise errors.ValidationError(name=_('trusted domain object'),
+                                             error=_('Object does not belong '
+                                                     'to a trusted domain'))
 
         # If fallback to AD DC LDAP is not allowed, bail out
         if not fallback_to_ldap:
@@ -491,9 +495,9 @@ class DomainValidator:
         # If unsuccessful, search AD DC LDAP
         logger.debug("Searching AD DC LDAP")
 
-        escaped_sid = escape_filter_chars(
-            security.dom_sid(sid).__ndr_pack__(),
-            2  # 2 means every character needs to be escaped
+        # escape_filter_chars(sid_bytes, 2) but for bytes
+        escaped_sid = "".join(
+            "\\%02x" % b for b in ndr_pack(security.dom_sid(sid))
         )
 
         attrs = ['sAMAccountName']
@@ -634,7 +638,7 @@ class DomainValidator:
             struct.unpack('<I', sid[8+4*i:12+4*i])[0]
             for i in range(number_sub_id)
         ]
-        return u'S-%d-%d-%s' % (sid_rev_num, ia,
+        return 'S-%d-%d-%s' % (sid_rev_num, ia,
                                 '-'.join([str(s) for s in subs]),)
 
     def kinit_as_administrator(self, domain):
@@ -821,7 +825,7 @@ class DomainValidator:
 
 def string_to_array(what):
     if six.PY3 and isinstance(what, bytes):
-        return [v for v in what]
+        return list(what)
     return [ord(v) for v in what]
 
 
@@ -900,9 +904,9 @@ class TrustDomainInstance:
 
         We try NCACN_NP before NCACN_IP_TCP and use SMB2 before SMB1.
         """
-        transports = (u'ncacn_np', u'ncacn_ip_tcp')
-        options = (u'smb2,print', u'print')
-        return [u'%s:%s[%s]' % (t, remote_host, o)
+        transports = ('ncacn_np', 'ncacn_ip_tcp')
+        options = ('smb2,print', 'print')
+        return ['%s:%s[%s]' % (t, remote_host, o)
                 for t in transports for o in options]
 
     def retrieve_anonymously(self, remote_host,
@@ -976,7 +980,7 @@ class TrustDomainInstance:
         objectAttribute.sec_qos = lsa.QosInfo()
         try:
             self._policy_handle = \
-                self._pipe.OpenPolicy2(u"", objectAttribute,
+                self._pipe.OpenPolicy2("", objectAttribute,
                                        security.SEC_FLAG_MAXIMUM_ALLOWED)
             result = self._pipe.QueryInfoPolicy2(self._policy_handle,
                                                  lsa.LSA_POLICY_INFO_DNS)
@@ -1301,7 +1305,7 @@ class TrustDomainInstance:
         """
         if self.info['name'] == another_domain.info['name']:
             # Check that NetBIOS names do not clash
-            raise errors.ValidationError(name=u'AD Trust Setup',
+            raise errors.ValidationError(name='AD Trust Setup',
                                          error=_('the IPA server and the '
                                                  'remote domain cannot share '
                                                  'the same NetBIOS name: %s')
@@ -1597,7 +1601,7 @@ def retrieve_remote_domain(hostname, local_flatname,
                 # realm admin is in DOMAIN\user format
                 # strip DOMAIN part as we'll enforce the one discovered
                 realm_admin = names[-1]
-            auth_string = u"%s\%s%%%s" \
+            auth_string = r"%s\%s%%%s" \
                           % (rd.info['name'], realm_admin, realm_passwd)
             td = get_instance(local_flatname)
             td.creds.parse_string(auth_string)
