@@ -32,18 +32,17 @@ import socket
 
 import six
 
-from ipaserver.dns_data_management import IPASystemRecords
 from ipaserver.install import service
 from ipaserver.install import installutils
-from ipaserver.install.bindinstance import dns_zone_exists
 from ipaserver.install.replication import wait_for_task
 from ipalib import errors, api
 from ipalib.util import normalize_zone
 from ipapython.dn import DN
+from ipapython import ipachangeconf
+from ipapython import ipaldap
 from ipapython import ipautil
 import ipapython.errors
 
-import ipaclient.install.ipachangeconf
 from ipaplatform import services
 from ipaplatform.constants import constants
 from ipaplatform.paths import paths
@@ -129,6 +128,15 @@ def map_Guests_to_nobody():
     logger.debug("Map BUILTIN\\Guests to a group 'nobody'")
     ipautil.run(args, env=env, raiseonerr=False, capture_error=True)
 
+
+def get_idmap_range(realm):
+    idrange = api.Command.idrange_show('{}_id_range'.format(realm))['result']
+    range_start = int(idrange['ipabaseid'][0])
+    range_size = int(idrange['ipaidrangesize'][0])
+    range_fmt = '{} - {}'.format(range_start, range_start + range_size)
+    return range_fmt
+
+
 class ADTRUSTInstance(service.Service):
 
     ATTR_SID = "ipaNTSecurityIdentifier"
@@ -178,7 +186,7 @@ class ADTRUSTInstance(service.Service):
 
         self.suffix = ipautil.realm_to_suffix(self.realm)
         self.ldapi_socket = "%%2fvar%%2frun%%2fslapd-%s.socket" % \
-                            installutils.realm_to_serverid(self.realm)
+                            ipaldap.realm_to_serverid(self.realm)
 
         # DN definitions
         self.trust_dn = DN(api.env.container_trusts, self.suffix)
@@ -557,7 +565,7 @@ class ADTRUSTInstance(service.Service):
         Purge old CIFS keys from samba and clean up samba ccache
         """
         self.clean_samba_keytab()
-        installutils.remove_ccache(paths.KRB5CC_SAMBA)
+        ipautil.remove_ccache(paths.KRB5CC_SAMBA)
 
     def set_keytab_owner(self, keytab=None, owner=None):
         """
@@ -576,43 +584,6 @@ class ADTRUSTInstance(service.Service):
                     logger.critical("Failed to remove old key for %s",
                                     self.principal)
 
-    def srv_rec(self, host, port, prio):
-        return "%(prio)d 100 %(port)d %(host)s" % dict(host=host,prio=prio,port=port)
-
-    def __add_dns_service_records(self):
-        """
-        Add DNS service records for Windows if DNS is enabled and the DNS zone
-        is managed. If there are already service records for LDAP and Kerberos
-        their values are used. Otherwise default values are used.
-        """
-
-        zone = api.env.domain
-
-        err_msg = None
-
-        ret = api.Command['dns_is_enabled']()
-        if not ret['result']:
-            err_msg = "DNS management was not enabled at install time."
-        else:
-            if not dns_zone_exists(zone):
-                err_msg = (
-                    "DNS zone %s cannot be managed as it is not defined in "
-                    "IPA" % zone)
-
-        if err_msg:
-            self.print_msg(err_msg)
-            self.print_msg("Add the following service records to your DNS " \
-                           "server for DNS zone %s: " % zone)
-            system_records = IPASystemRecords(api, all_servers=True)
-            adtrust_records = system_records.get_base_records(
-                [self.fqdn], ["AD trust controller"],
-                include_master_role=False, include_kerberos_realm=False)
-            for r_name, node in adtrust_records.items():
-                for rec in IPASystemRecords.records_list_from_node(r_name, node):
-                    self.print_msg(rec)
-        else:
-            api.Command.dns_update_system_records()
-
     def __configure_selinux_for_smbd(self):
         try:
             tasks.set_selinux_booleans(constants.SELINUX_BOOLEAN_ADTRUST,
@@ -629,7 +600,7 @@ class ADTRUSTInstance(service.Service):
             self.print_msg("Cannot modify /etc/krb5.conf")
 
         krbconf = (
-            ipaclient.install.ipachangeconf.IPAChangeConf("IPA Installer"))
+            ipachangeconf.IPAChangeConf("IPA Installer"))
         krbconf.setOptionAssignment((" = ", " "))
         krbconf.setSectionNameDelimiters(("[", "]"))
         krbconf.setSubSectionDelimiters(("{", "}"))
@@ -665,7 +636,8 @@ class ADTRUSTInstance(service.Service):
 
         has_dns_lookup_kdc_true = False
         for line in krb5conf:
-            if re.match("^\s*dns_lookup_kdc\s*=\s*[Tt][Rr][Uu][Ee]\s*$", line):
+            regex = r"^\s*dns_lookup_kdc\s*=\s*[Tt][Rr][Uu][Ee]\s*$"
+            if re.match(regex, line):
                 has_dns_lookup_kdc_true = True
                 break
         krb5conf.close()
@@ -837,12 +809,18 @@ class ADTRUSTInstance(service.Service):
         )
         api.Backend.ldap2.add_entry(entry)
 
+    def __retrieve_local_range(self):
+        """Retrieves local IPA ID range to make sure
+        """
+        self.sub_dict['IPA_LOCAL_RANGE'] = get_idmap_range(self.realm)
+
     def create_instance(self):
         self.step("validate server hostname",
                   self.__validate_server_hostname)
         self.step("stopping smbd", self.__stop)
         self.step("creating samba domain object", \
                   self.__create_samba_domain_object)
+        self.step("retrieve local idmap range", self.__retrieve_local_range)
         self.step("creating samba config registry", self.__write_smb_registry)
         self.step("writing samba config file", self.__write_smb_conf)
         self.step("adding cifs Kerberos principal",
@@ -859,8 +837,6 @@ class ADTRUSTInstance(service.Service):
         self.step("map BUILTIN\\Guests to nobody group",
                   self.__map_Guests_to_nobody)
         self.step("configuring smbd to start on boot", self.__enable)
-        self.step("adding special DNS service records", \
-                  self.__add_dns_service_records)
 
         if self.enable_compat:
             self.step("enabling trusted domains support for older clients via Schema Compatibility plugin",
@@ -913,16 +889,16 @@ class ADTRUSTInstance(service.Service):
             self.print_msg('WARNING: ' + str(e))
 
         # Remove samba's credentials cache
-        installutils.remove_ccache(ccache_path=paths.KRB5CC_SAMBA)
+        ipautil.remove_ccache(ccache_path=paths.KRB5CC_SAMBA)
 
         # Remove samba's configuration file
-        installutils.remove_file(self.smb_conf)
+        ipautil.remove_file(self.smb_conf)
 
         # Remove samba's persistent and temporary tdb files
         tdb_files = [tdb_file for tdb_file in os.listdir(paths.SAMBA_DIR)
                                            if tdb_file.endswith(".tdb")]
         for tdb_file in tdb_files:
-            installutils.remove_file(tdb_file)
+            ipautil.remove_file(tdb_file)
 
         # Remove our keys from samba's keytab
         self.clean_samba_keytab()
