@@ -40,6 +40,8 @@ from ipaserver.dns_data_management import (
 from ipaserver.install import installutils
 from ipaserver.install import service
 from ipaserver.install import sysupgrade
+from ipaserver.masters import get_masters
+from ipapython import ipaldap
 from ipapython import ipautil
 from ipapython import dnsutil
 from ipapython.dnsutil import DNSName
@@ -49,6 +51,7 @@ import ipalib
 from ipalib import api, errors
 from ipalib.constants import IPA_CA_RECORD
 from ipaplatform import services
+from ipaplatform.tasks import tasks
 from ipaplatform.constants import constants
 from ipaplatform.paths import paths
 from ipalib.util import (validate_zonemgr_str, normalize_zonemgr,
@@ -64,8 +67,8 @@ if six.PY3:
 
 logger = logging.getLogger(__name__)
 
-named_conf_section_ipa_start_re = re.compile('\s*dyndb\s+"ipa"\s+"[^"]+"\s+{')
-named_conf_section_options_start_re = re.compile('\s*options\s+{')
+named_conf_section_ipa_start_re = re.compile(r'\s*dyndb\s+"ipa"\s+"[^"]+"\s+{')
+named_conf_section_options_start_re = re.compile(r'\s*options\s+{')
 named_conf_section_end_re = re.compile('};')
 named_conf_arg_ipa_re = re.compile(
     r'(?P<indent>\s*)(?P<name>\S+)\s"(?P<value>[^"]+)";')
@@ -201,7 +204,7 @@ def named_conf_set_directive(name, value, section=NAMED_SECTION_IPA,
                     if name == match.group('name'):
                         matched = True
                         if value is not None:
-                            if not isinstance(value, six.string_types):
+                            if not isinstance(value, str):
                                 value = str(value)
                             new_conf = named_conf_arg_template \
                                     % dict(indent=last_indent,
@@ -292,6 +295,26 @@ def find_reverse_zone(ip_address, api=api):
     return None
 
 
+def named_add_ext_conf_file(src, dest, t_params={}):
+    """
+    Ensure included file is present, but don't override it.
+
+    :param src: String. Absolute path to source template
+    :param dest: String. Absolute path to destination
+    :param t_params: Dict. Parameters for source template
+    """
+    if not os.path.exists(dest):
+        ipa_ext_txt = ipautil.template_file(src, t_params)
+        gid = pwd.getpwnam(constants.NAMED_USER).pw_gid
+
+        with open(dest, 'w') as ipa_ext:
+            os.fchmod(ipa_ext.fileno(), 0o640)
+            os.fchown(ipa_ext.fileno(), 0, gid)
+            ipa_ext.write(ipa_ext_txt)
+        return True
+    return False
+
+
 def read_reverse_zone(default, ip_address, allow_zone_overlap=False):
     while True:
         zone = ipautil.user_input("Please specify the reverse zone name", default=default)
@@ -304,7 +327,7 @@ def read_reverse_zone(default, ip_address, allow_zone_overlap=False):
         if not allow_zone_overlap:
             try:
                 dnsutil.check_zone_overlap(zone, raise_on_error=False)
-            except ValueError as e:
+            except dnsutil.DNSZoneAlreadyExists as e:
                 logger.error("Reverse zone %s will not be used: %s",
                              zone, e)
                 continue
@@ -324,9 +347,10 @@ def get_auto_reverse_zones(ip_addresses, allow_zone_overlap=False):
         if not allow_zone_overlap:
             try:
                 dnsutil.check_zone_overlap(default_reverse)
-            except ValueError:
+            except ValueError as e:
                 logger.info("Reverse zone %s for IP address %s already exists",
                             default_reverse, ip)
+                logger.debug('%s', e)
                 continue
         auto_zones.append((ip, default_reverse))
     return auto_zones
@@ -585,7 +609,7 @@ def ensure_dnsserver_container_exists(ldap, api_instance, logger=logger):
         logger.debug('cn=servers,cn=dns container already exists')
 
 
-class DnsBackup(object):
+class DnsBackup:
     def __init__(self, service):
         self.service = service
         self.zones = {}
@@ -803,7 +827,7 @@ class BindInstance(service.Service):
 
         self.sub_dict = dict(
             FQDN=self.fqdn,
-            SERVER_ID=installutils.realm_to_serverid(self.realm),
+            SERVER_ID=ipaldap.realm_to_serverid(self.realm),
             SUFFIX=self.suffix,
             BINDKEYS_FILE=paths.NAMED_BINDKEYS_FILE,
             MANAGED_KEYS_DIR=paths.NAMED_MANAGED_KEYS_DIR,
@@ -814,6 +838,7 @@ class BindInstance(service.Service):
             NAMED_VAR_DIR=paths.NAMED_VAR_DIR,
             BIND_LDAP_SO=paths.BIND_LDAP_SO,
             INCLUDE_CRYPTO_POLICY=crypto_policy,
+            CUSTOM_CONFIG=paths.NAMED_CUSTOM_CONFIG,
             NAMED_DATA_DIR=constants.NAMED_DATA_DIR,
             NAMED_ZONE_COMMENT=constants.NAMED_ZONE_COMMENT,
         )
@@ -880,10 +905,13 @@ class BindInstance(service.Service):
 
         # Add forward and reverse records to self
         for addr in addrs:
-            try:
+            # Check first if the zone is a master zone
+            # (if it is a forward zone, dns_zone_exists will return False)
+            if dns_zone_exists(zone, api=self.api):
                 add_fwd_rr(zone, host, addr, self.api)
-            except errors.NotFound:
-                pass
+            else:
+                logger.debug("Skip adding record %s to a zone %s "
+                             "not managed by IPA", addr, zone)
 
             reverse_zone = find_reverse_zone(addr, self.api)
             if reverse_zone:
@@ -894,8 +922,7 @@ class BindInstance(service.Service):
 
     def __add_others(self):
         entries = api.Backend.ldap2.get_entries(
-            DN(('cn', 'masters'), ('cn', 'ipa'), ('cn', 'etc'),
-               self.suffix),
+            DN(api.env.container_masters, self.suffix),
             api.Backend.ldap2.SCOPE_ONELEVEL, None, ['dn'])
 
         for entry in entries:
@@ -969,6 +996,9 @@ class BindInstance(service.Service):
         named_fd.write(named_txt)
         named_fd.close()
 
+        named_add_ext_conf_file(paths.NAMED_CUSTOM_CFG_SRC,
+                                paths.NAMED_CUSTOM_CONFIG)
+
         if self.no_dnssec_validation:
             # disable validation
             named_conf_set_directive("dnssec-validation", "no",
@@ -1003,32 +1033,25 @@ class BindInstance(service.Service):
         sysupgrade.set_upgrade_state('dns', 'server_config_to_ldap', True)
 
     def __setup_resolv_conf(self):
-        if not self.fstore.has_file(paths.RESOLV_CONF):
-            self.fstore.backup_file(paths.RESOLV_CONF)
-
-        resolv_txt = "search "+self.domain+"\n"
+        searchdomains = [self.domain]
+        nameservers = []
 
         for ip_address in self.ip_addresses:
             if ip_address.version == 4:
-                resolv_txt += "nameserver 127.0.0.1\n"
-                break
+                nameservers.append("127.0.0.1")
+            elif ip_address.version == 6:
+                nameservers.append("::1")
 
-        for ip_address in self.ip_addresses:
-            if ip_address.version == 6:
-                resolv_txt += "nameserver ::1\n"
-                break
         try:
-            resolv_fd = open(paths.RESOLV_CONF, 'w')
-            resolv_fd.seek(0)
-            resolv_fd.truncate(0)
-            resolv_fd.write(resolv_txt)
-            resolv_fd.close()
+            tasks.configure_dns_resolver(
+                nameservers, searchdomains, fstore=self.fstore
+            )
         except IOError as e:
-            logger.error('Could not write to resolv.conf: %s', e)
+            logger.error('Could not update DNS config: %s', e)
         else:
             # python DNS might have global resolver cached in this variable
             # we have to re-initialize it because resolv.conf has changed
-            dns.resolver.default_resolver = None
+            dns.resolver.reset_default_resolver()
 
     def __generate_rndc_key(self):
         installutils.check_entropy()
@@ -1070,13 +1093,8 @@ class BindInstance(service.Service):
             cname_fqdn[cname] = fqdn
 
         # get FQDNs of all IPA masters
-        ldap = self.api.Backend.ldap2
         try:
-            entries = ldap.get_entries(
-                DN(('cn', 'masters'), ('cn', 'ipa'), ('cn', 'etc'),
-                   self.api.env.basedn),
-                ldap.SCOPE_ONELEVEL, None, ['cn'])
-            masters = set(e['cn'][0] for e in entries)
+            masters = set(get_masters(self.api.Backend.ldap2))
         except errors.NotFound:
             masters = set()
 
@@ -1099,6 +1117,10 @@ class BindInstance(service.Service):
         self.fqdn = fqdn
         self.domain = domain_name
 
+        if not dns_zone_exists(zone, api=self.api):
+            # Zone may be a forward zone, skip update
+            return
+
         areclist = get_fwd_rr(zone, host, api=self.api)
         for rdata in areclist:
             del_fwd_rr(zone, host, rdata, api=self.api)
@@ -1209,13 +1231,17 @@ class BindInstance(service.Service):
 
         self.dns_backup.clear_records(self.api.Backend.ldap2.isconnected())
 
-        for f in [paths.NAMED_CONF, paths.RESOLV_CONF]:
-            try:
-                self.fstore.restore_file(f)
-            except ValueError as error:
-                logger.debug('%s', error)
+        try:
+            self.fstore.restore_file(paths.NAMED_CONF)
+        except ValueError as error:
+            logger.debug('%s', error)
 
-        installutils.rmtree(paths.BIND_LDAP_DNS_IPA_WORKDIR)
+        try:
+            tasks.unconfigure_dns_resolver(fstore=self.fstore)
+        except Exception:
+            logger.exception("Failed to unconfigure DNS resolver")
+
+        ipautil.rmtree(paths.BIND_LDAP_DNS_IPA_WORKDIR)
 
         # disabled by default, by ldap_configure()
         if enabled:
@@ -1235,5 +1261,6 @@ class BindInstance(service.Service):
         if named_regular_running:
             self.named_regular.start()
 
-        installutils.remove_keytab(self.keytab)
-        installutils.remove_ccache(run_as=self.service_user)
+        ipautil.remove_file(paths.NAMED_CUSTOM_CONFIG)
+        ipautil.remove_keytab(self.keytab)
+        ipautil.remove_ccache(run_as=self.service_user)
