@@ -2,7 +2,9 @@
 # Copyright (C) 2014  FreeIPA Contributors see COPYING for license
 #
 
-import random
+from __future__ import absolute_import
+
+import logging
 import os
 import pwd
 import grp
@@ -10,20 +12,23 @@ import stat
 import shutil
 from subprocess import CalledProcessError
 
-from ipapython import p11helper as _ipap11helper
+from ipalib.install import sysrestore
+from ipaserver.dnssec.opendnssec import tasks
 from ipaserver.install import service
-from ipaserver.install import installutils
-from ipapython.ipa_log_manager import *
+from ipaserver.masters import ENABLED_SERVICE
 from ipapython.dn import DN
-from ipapython import sysrestore, ipautil, ipaldap, p11helper
+from ipapython import directivesetter
+from ipapython import ipautil
 from ipaplatform import services
 from ipaplatform.constants import constants
 from ipaplatform.paths import paths
 from ipalib import errors, api
-from ipaserver.install import dnskeysyncinstance
+from ipaserver import p11helper
+from ipalib.constants import SOFTHSM_DNSSEC_TOKEN_LABEL
+
+logger = logging.getLogger(__name__)
 
 KEYMASTER = u'dnssecKeyMaster'
-softhsm_slot = 0
 
 
 def get_dnssec_key_masters(conn):
@@ -42,7 +47,7 @@ def get_dnssec_key_masters(conn):
     filter_attrs = {
         u'cn': u'DNSSEC',
         u'objectclass': u'ipaConfigObject',
-        u'ipaConfigString': [KEYMASTER, u'enabledService'],
+        u'ipaConfigString': [KEYMASTER, ENABLED_SERVICE],
     }
     only_masters_f = conn.make_filter(filter_attrs, rules=conn.MATCH_ALL)
 
@@ -59,22 +64,16 @@ def get_dnssec_key_masters(conn):
 
 
 class OpenDNSSECInstance(service.Service):
-    def __init__(self, fstore=None, dm_password=None, ldapi=False,
-                 start_tls=False, autobind=ipaldap.AUTOBIND_ENABLED):
+    def __init__(self, fstore=None):
         service.Service.__init__(
             self, "ods-enforcerd",
             service_desc="OpenDNSSEC enforcer daemon",
-            dm_password=dm_password,
-            ldapi=ldapi,
-            autobind=autobind,
-            start_tls=start_tls
         )
-        self.dm_password = dm_password
         self.ods_uid = None
         self.ods_gid = None
         self.conf_file_dict = {
             'SOFTHSM_LIB': paths.LIBSOFTHSM2_SO,
-            'TOKEN_LABEL': dnskeysyncinstance.softhsm_token_label,
+            'TOKEN_LABEL': SOFTHSM_DNSSEC_TOKEN_LABEL,
             'KASP_DB': paths.OPENDNSSEC_KASP_DB,
             'ODS_USER': constants.ODS_USER,
             'ODS_GROUP': constants.ODS_GROUP,
@@ -90,9 +89,7 @@ class OpenDNSSECInstance(service.Service):
     suffix = ipautil.dn_attribute_property('_suffix')
 
     def get_masters(self):
-        if not self.admin_conn:
-            self.ldap_connect()
-        return get_dnssec_key_masters(self.admin_conn)
+        return get_dnssec_key_masters(api.Backend.ldap2)
 
     def create_instance(self, fqdn, realm_name, generate_master_key=True,
                         kasp_db_file=None):
@@ -110,9 +107,6 @@ class OpenDNSSECInstance(service.Service):
         except Exception:
             pass
 
-        # get a connection to the DS
-        if not self.admin_conn:
-            self.ldap_connect()
         # checking status must be first
         self.step("checking status", self.__check_dnssec_status)
         self.step("setting up configuration files", self.__setup_conf_files)
@@ -126,9 +120,6 @@ class OpenDNSSECInstance(service.Service):
         self.start_creation()
 
     def __check_dnssec_status(self):
-        named = services.knownservices.named
-        ods_enforcerd = services.knownservices.ods_enforcerd
-
         try:
             self.named_uid = pwd.getpwnam(constants.NAMED_USER).pw_uid
         except KeyError:
@@ -151,25 +142,25 @@ class OpenDNSSECInstance(service.Service):
 
     def __enable(self):
         try:
-            self.ldap_enable('DNSSEC', self.fqdn, self.dm_password,
-                             self.suffix, self.extra_config)
+            self.ldap_configure('DNSSEC', self.fqdn, None,
+                                self.suffix, self.extra_config)
         except errors.DuplicateEntry:
-            root_logger.error("DNSSEC service already exists")
+            logger.error("DNSSEC service already exists")
 
         # add the KEYMASTER identifier into ipaConfigString
         # this is needed for the re-enabled DNSSEC master
         dn = DN(('cn', 'DNSSEC'), ('cn', self.fqdn), api.env.container_masters,
                 api.env.basedn)
         try:
-            entry = self.admin_conn.get_entry(dn, ['ipaConfigString'])
+            entry = api.Backend.ldap2.get_entry(dn, ['ipaConfigString'])
         except errors.NotFound as e:
-            root_logger.error(
+            logger.error(
                 "DNSSEC service entry not found in the LDAP (%s)", e)
         else:
             config = entry.setdefault('ipaConfigString', [])
             if KEYMASTER not in config:
                 config.append(KEYMASTER)
-                self.admin_conn.update_entry(entry)
+                api.Backend.ldap2.update_entry(entry)
 
     def __setup_conf_files(self):
         if not self.fstore.has_file(paths.OPENDNSSEC_CONF_FILE):
@@ -188,9 +179,16 @@ class OpenDNSSECInstance(service.Service):
         # add pin to template
         sub_conf_dict = self.conf_file_dict
         sub_conf_dict['PIN'] = pin
+        if paths.ODS_KSMUTIL is not None and os.path.exists(paths.ODS_KSMUTIL):
+            # OpenDNSSEC 1.4
+            sub_conf_dict['INTERVAL'] = '<Interval>PT3600S</Interval>'
+        else:
+            # OpenDNSSEC 2.x
+            sub_conf_dict['INTERVAL'] = '<!-- Interval not used in 2x -->'
 
         ods_conf_txt = ipautil.template_file(
-            ipautil.SHARE_DIR + "opendnssec_conf.template", sub_conf_dict)
+            os.path.join(paths.USR_SHARE_IPA_DIR, "opendnssec_conf.template"),
+            sub_conf_dict)
         ods_conf_fd = open(paths.OPENDNSSEC_CONF_FILE, 'w')
         ods_conf_fd.seek(0)
         ods_conf_fd.truncate(0)
@@ -198,7 +196,8 @@ class OpenDNSSECInstance(service.Service):
         ods_conf_fd.close()
 
         ods_kasp_txt = ipautil.template_file(
-            ipautil.SHARE_DIR + "opendnssec_kasp.template", self.kasp_file_dict)
+            os.path.join(paths.USR_SHARE_IPA_DIR, "opendnssec_kasp.template"),
+            self.kasp_file_dict)
         ods_kasp_fd = open(paths.OPENDNSSEC_KASP_FILE, 'w')
         ods_kasp_fd.seek(0)
         ods_kasp_fd.truncate(0)
@@ -208,10 +207,15 @@ class OpenDNSSECInstance(service.Service):
         if not self.fstore.has_file(paths.SYSCONFIG_ODS):
             self.fstore.backup_file(paths.SYSCONFIG_ODS)
 
-        installutils.set_directive(paths.SYSCONFIG_ODS,
-                                   'SOFTHSM2_CONF',
-                                    paths.DNSSEC_SOFTHSM2_CONF,
-                                    quotes=False, separator='=')
+        if not os.path.isfile(paths.SYSCONFIG_ODS):
+            # create file, it's not shipped on Debian
+            with open(paths.SYSCONFIG_ODS, 'a') as f:
+                os.fchmod(f.fileno(), 0o644)
+
+        directivesetter.set_directive(paths.SYSCONFIG_ODS,
+                                      'SOFTHSM2_CONF',
+                                      paths.DNSSEC_SOFTHSM2_CONF,
+                                      quotes=False, separator='=')
 
     def __setup_ownership_file_modes(self):
         assert self.ods_uid is not None
@@ -249,14 +253,15 @@ class OpenDNSSECInstance(service.Service):
             pin = f.read()
 
         os.environ["SOFTHSM2_CONF"] = paths.DNSSEC_SOFTHSM2_CONF
-        p11 = _ipap11helper.P11_Helper(softhsm_slot, pin, paths.LIBSOFTHSM2_SO)
+        p11 = p11helper.P11_Helper(
+            SOFTHSM_DNSSEC_TOKEN_LABEL, pin, paths.LIBSOFTHSM2_SO)
         try:
             # generate master key
-            root_logger.debug("Creating master key")
+            logger.debug("Creating master key")
             p11helper.generate_master_key(p11)
 
             # change tokens mod/owner
-            root_logger.debug("Changing ownership of token files")
+            logger.debug("Changing ownership of token files")
             for (root, dirs, files) in os.walk(paths.DNSSEC_TOKENS_DIR):
                 for directory in dirs:
                     dir_path = os.path.join(root, directory)
@@ -273,7 +278,7 @@ class OpenDNSSECInstance(service.Service):
     def __setup_dnssec(self):
         # run once only
         if self.get_state("kasp_db_configured") and not self.kasp_db_file:
-            root_logger.debug("Already configured, skipping step")
+            logger.debug("Already configured, skipping step")
             return
 
         self.backup_state("kasp_db_configured", True)
@@ -288,37 +293,29 @@ class OpenDNSSECInstance(service.Service):
             os.chown(paths.OPENDNSSEC_KASP_DB, self.ods_uid, self.ods_gid)
             os.chmod(paths.OPENDNSSEC_KASP_DB, 0o660)
 
-            # regenerate zonelist.xml
-            ods_enforcerd = services.knownservices.ods_enforcerd
-            cmd = [paths.ODS_KSMUTIL, 'zonelist', 'export']
-            result = ipautil.run(cmd,
-                                 runas=constants.ODS_USER,
-                                 capture_output=True)
-            with open(paths.OPENDNSSEC_ZONELIST_FILE, 'w') as zonelistf:
-                zonelistf.write(result.output)
-                os.chown(paths.OPENDNSSEC_ZONELIST_FILE,
-                         self.ods_uid, self.ods_gid)
-                os.chmod(paths.OPENDNSSEC_ZONELIST_FILE, 0o660)
-
         else:
             # initialize new kasp.db
-            command = [
-                paths.ODS_KSMUTIL,
-                'setup'
-            ]
-
-            ods_enforcerd = services.knownservices.ods_enforcerd
-            ipautil.run(command, stdin="y", runas=constants.ODS_USER)
+            tasks.run_ods_setup()
 
     def __setup_dnskeysyncd(self):
         # set up dnskeysyncd this is DNSSEC master
-        installutils.set_directive(paths.SYSCONFIG_IPA_DNSKEYSYNCD,
-                                   'ISMASTER',
-                                   '1',
-                                   quotes=False, separator='=')
+        directivesetter.set_directive(paths.SYSCONFIG_IPA_DNSKEYSYNCD,
+                                      'ISMASTER',
+                                      '1',
+                                      quotes=False, separator='=')
 
     def __start(self):
         self.restart()  # needed to reload conf files
+        tasks.run_ods_policy_import()
+        if self.kasp_db_file:
+            # regenerate zonelist.xml
+            result = tasks.run_ods_manager(
+                ['zonelist', 'export'], capture_output=True
+            )
+            with open(paths.OPENDNSSEC_ZONELIST_FILE, 'w') as f:
+                f.write(result.output)
+                os.fchown(f.fileno(), self.ods_uid, self.ods_gid)
+                os.fchmod(f.fileno(), 0o660)
 
     def uninstall(self):
         if not self.is_configured():
@@ -335,7 +332,7 @@ class OpenDNSSECInstance(service.Service):
         except Exception:
             pass
 
-        ods_exporter = services.service('ipa-ods-exporter')
+        ods_exporter = services.service('ipa-ods-exporter', api)
         try:
             ods_exporter.stop()
         except Exception:
@@ -343,34 +340,33 @@ class OpenDNSSECInstance(service.Service):
 
         # remove directive from ipa-dnskeysyncd, this server is not DNSSEC
         # master anymore
-        installutils.set_directive(paths.SYSCONFIG_IPA_DNSKEYSYNCD,
-                                   'ISMASTER', None,
-                                   quotes=False, separator='=')
+        directivesetter.set_directive(paths.SYSCONFIG_IPA_DNSKEYSYNCD,
+                                      'ISMASTER', None,
+                                      quotes=False, separator='=')
 
         restore_list = [paths.OPENDNSSEC_CONF_FILE, paths.OPENDNSSEC_KASP_FILE,
                         paths.SYSCONFIG_ODS, paths.OPENDNSSEC_ZONELIST_FILE]
 
-        if ipautil.file_exists(paths.OPENDNSSEC_KASP_DB):
+        if os.path.isfile(paths.OPENDNSSEC_KASP_DB):
 
             # force to export data
-            ods_enforcerd = services.knownservices.ods_enforcerd
             cmd = [paths.IPA_ODS_EXPORTER, 'ipa-full-update']
             try:
                 self.print_msg("Exporting DNSSEC data before uninstallation")
                 ipautil.run(cmd, runas=constants.ODS_USER)
             except CalledProcessError:
-                root_logger.error("DNSSEC data export failed")
+                logger.error("DNSSEC data export failed")
 
             try:
                 shutil.copy(paths.OPENDNSSEC_KASP_DB,
                             paths.IPA_KASP_DB_BACKUP)
             except IOError as e:
-                root_logger.error(
+                logger.error(
                     "Unable to backup OpenDNSSEC database %s, "
                     "restore will be skipped: %s", paths.OPENDNSSEC_KASP_DB, e)
             else:
-                root_logger.info("OpenDNSSEC database backed up in %s",
-                                 paths.IPA_KASP_DB_BACKUP)
+                logger.info("OpenDNSSEC database backed up in %s",
+                            paths.IPA_KASP_DB_BACKUP)
                 # restore OpenDNSSEC's KASP DB only if backup succeeded
                 # removing the file without backup could totally break DNSSEC
                 restore_list.append(paths.OPENDNSSEC_KASP_DB)
@@ -379,12 +375,11 @@ class OpenDNSSECInstance(service.Service):
             try:
                 self.fstore.restore_file(f)
             except ValueError as error:
-                root_logger.debug(error)
-                pass
+                logger.debug("%s", error)
 
         self.restore_state("kasp_db_configured")  # just eat state
 
-        # disabled by default, by ldap_enable()
+        # disabled by default, by ldap_configure()
         if enabled:
             self.enable()
 
