@@ -137,8 +137,7 @@ def enable_replication_version_checking(realm, dirman_passwd):
     enabled then enable it and restart 389-ds. If it is enabled
     the do nothing.
     """
-    ldap_uri = ipaldap.get_ldap_uri(protocol='ldapi', realm=realm)
-    conn = ipaldap.LDAPClient(ldap_uri)
+    conn = ipaldap.LDAPClient.from_realm(realm)
     if dirman_passwd:
         conn.simple_bind(bind_dn=ipaldap.DIRMAN_DN,
                          bind_password=dirman_passwd)
@@ -187,7 +186,8 @@ def wait_for_entry(connection, dn, timeout, attr=None, attrvalue='*',
         attrlist.append(attr)
     else:
         filterstr = "(objectclass=*)"
-    log("Waiting for replication (%s) %s %s", connection, dn, filterstr)
+    log("Waiting up to %s seconds for replication (%s) %s %s",
+        timeout, connection, dn, filterstr)
     entry = []
     deadline = time.time() + timeout
     for i in itertools.count(start=1):
@@ -215,7 +215,23 @@ def wait_for_entry(connection, dn, timeout, attr=None, attrvalue='*',
             time.sleep(1)
 
 
-class ReplicationManager(object):
+def get_ds_version(conn):
+    """Returns the DS version
+
+    Retrieves the DS version from the vendorVersion attribute stored in LDAP.
+    :param conn: LDAP connection established and authenticated to the server
+                 for which we need the version
+    :return: a tuple containing the DS version
+    """
+    # Find which 389-ds is installed
+    rootdse = conn.get_entry(DN(''), ['vendorVersion'])
+    version = rootdse.single_value.get('vendorVersion')
+    mo = re.search(r'(\d+)\.(\d+)\.(\d+)[\.\d]*', version)
+    vendor_version = tuple(int(v) for v in mo.groups())
+    return vendor_version
+
+
+class ReplicationManager:
     """Manage replication agreements
 
     between DS servers, and sync  agreements with Windows servers
@@ -527,8 +543,16 @@ class ReplicationManager(object):
             # Add the new replication manager
             binddns.append(replica_binddn)
 
-        for key, value in REPLICA_CREATION_SETTINGS.items():
-            entry[key] = value
+        # If the remote server has 389-ds < 1.3, it does not
+        # support the attributes we are trying to set.
+        # Find which 389-ds is installed
+        vendor_version = get_ds_version(conn)
+        if vendor_version >= (1, 3, 0):
+            for key, value in REPLICA_CREATION_SETTINGS.items():
+                entry[key] = value
+        else:
+            logger.debug("replication attributes not supported "
+                         "on remote master, skipping update.")
 
         try:
             conn.update_entry(entry)
@@ -595,8 +619,9 @@ class ReplicationManager(object):
         """
         self._finalize_replica_settings(self.conn)
 
-        ldap_uri = ipaldap.get_ldap_uri(r_hostname)
-        r_conn = ipaldap.LDAPClient(ldap_uri, cacert=cacert)
+        r_conn = ipaldap.LDAPClient.from_hostname_secure(
+            r_hostname, cacert=cacert
+        )
         if r_bindpw:
             r_conn.simple_bind(r_binddn, r_bindpw)
         else:
@@ -604,10 +629,7 @@ class ReplicationManager(object):
         # If the remote server has 389-ds < 1.3, it does not
         # support the attributes we are trying to set.
         # Find which 389-ds is installed
-        rootdse = r_conn.get_entry(DN(''), ['vendorVersion'])
-        version = rootdse.single_value.get('vendorVersion')
-        mo = re.search(r'(\d+)\.(\d+)\.(\d+)[\.\d]*', version)
-        vendor_version = tuple(int(v) for v in mo.groups())
+        vendor_version = get_ds_version(r_conn)
         if vendor_version >= (1, 3, 0):
             # 389-ds understands the replication attributes,
             # we can safely modify them
@@ -1049,11 +1071,23 @@ class ReplicationManager(object):
             inprogress = entry.single_value.get('nsds5replicaUpdateInProgress')
             status = entry.single_value.get('nsds5ReplicaLastUpdateStatus')
             try:
-                start = int(entry.single_value['nsds5ReplicaLastUpdateStart'])
+                # nsds5ReplicaLastUpdateStart is either a GMT time
+                # ending with Z or 0 (see 389-ds ticket 47836)
+                # Remove the Z and convert to int
+                start = entry.single_value['nsds5ReplicaLastUpdateStart']
+                if start.endswith('Z'):
+                    start = start[:-1]
+                start = int(start)
             except (ValueError, TypeError, KeyError):
                 start = 0
             try:
-                end = int(entry.single_value['nsds5ReplicaLastUpdateEnd'])
+                # nsds5ReplicaLastUpdateEnd is either a GMT time
+                # ending with Z or 0 (see 389-ds ticket 47836)
+                # Remove the Z and convert to int
+                end = entry.single_value['nsds5ReplicaLastUpdateEnd']
+                if end.endswith('Z'):
+                    end = end[:-1]
+                end = int(end)
             except (ValueError, TypeError, KeyError):
                 end = 0
             # incremental update is done if inprogress is false and end >= start
@@ -1127,12 +1161,7 @@ class ReplicationManager(object):
             local_port = r_port
         # note - there appears to be a bug in python-ldap - it does not
         # allow connections using two different CA certs
-        ldap_uri = ipaldap.get_ldap_uri(r_hostname, r_port,
-                                        cacert=paths.IPA_CA_CRT,
-                                        protocol='ldap')
-        r_conn = ipaldap.LDAPClient(ldap_uri,
-                                    cacert=paths.IPA_CA_CRT,
-                                    start_tls=True)
+        r_conn = ipaldap.LDAPClient.from_hostname_secure(r_hostname)
 
         if r_bindpw:
             r_conn.simple_bind(r_binddn, r_bindpw)
@@ -1171,6 +1200,46 @@ class ReplicationManager(object):
         if ret != 0:
             raise RuntimeError("Failed to start replication")
 
+    def unhashed_password_log(self, conn, value):
+        if not conn:
+            raise RuntimeError("unhashed_password_log no connection")
+
+        if not value:
+            return
+
+        # Check the validity of the value
+        if value.lower() not in ['off', 'on', 'nolog']:
+            return
+
+        # Change the value if needed
+        entry = conn.get_entry(DN(('cn', 'config')),
+                               ['nsslapd-unhashed-pw-switch'])
+        toggle = entry.single_value.get("nsslapd-unhashed-pw-switch")
+        if not toggle or not (toggle.lower() == value.lower()):
+            entry["nsslapd-unhashed-pw-switch"] = value.lower()
+            conn.update_entry(entry)
+
+        # if unhashed password are being logged, display a warning
+        if value.lower() == 'on':
+            try:
+                entry = conn.get_entry(
+                    DN(('cn', 'changelog5'),
+                       ('cn', 'config')),
+                    ['nsslapd-changelogdir'])
+                cldb = entry.single_value.get("nsslapd-changelogdir")
+                logger.warning("This configuration (\"--winsync\") may imply "
+                               "that the log file contains clear text "
+                               "passwords.\n"
+                               "Please ensure that these files can be accessed"
+                               " only by trusted accounts.\n"
+                               "Log files are under %s", cldb)
+            except errors.NotFound:
+                logger.warning("This configuration (\"--winsync\") may imply "
+                               "that the log file contains clear text "
+                               "passwords.\n"
+                               "Please ensure that these files can be accessed"
+                               " only by trusted accounts.")
+
     def setup_winsync_replication(self,
                                   ad_dc_name, ad_binddn, ad_pwd,
                                   passsync_pw, ad_subtree,
@@ -1232,15 +1301,18 @@ class ReplicationManager(object):
         except Exception as e:
             logger.info("Failed to create public entry for winsync replica")
 
+        # For winsync, unhashed passwords needs to be in replication changelog
+        # Time to alarm about a security risk
+        self.unhashed_password_log(self.conn, 'on')
+
+
         #Finally start replication
         ret = self.start_replication(self.conn, ad_dc_name)
         if ret != 0:
             raise RuntimeError("Failed to start replication")
 
     def convert_to_gssapi_replication(self, r_hostname, r_binddn, r_bindpw):
-        ldap_uri = ipaldap.get_ldap_uri(r_hostname, PORT,
-                                        cacert=paths.IPA_CA_CRT)
-        r_conn = ipaldap.LDAPClient(ldap_uri, cacert=paths.IPA_CA_CRT)
+        r_conn = ipaldap.LDAPClient.from_hostname_secure(r_hostname)
         if r_bindpw:
             r_conn.simple_bind(r_binddn, r_bindpw)
         else:
@@ -1268,11 +1340,7 @@ class ReplicationManager(object):
         Only usable to connect 2 existing replicas (needs existing kerberos
         principals)
         """
-        # note - there appears to be a bug in python-ldap - it does not
-        # allow connections using two different CA certs
-        ldap_uri = ipaldap.get_ldap_uri(r_hostname, PORT,
-                                        cacert=paths.IPA_CA_CRT)
-        r_conn = ipaldap.LDAPClient(ldap_uri, cacert=paths.IPA_CA_CRT)
+        r_conn = ipaldap.LDAPClient.from_hostname_secure(r_hostname)
         if r_bindpw:
             r_conn.simple_bind(r_binddn, r_bindpw)
         else:
@@ -1398,8 +1466,7 @@ class ReplicationManager(object):
 
         # delete master entry with all active services
         try:
-            dn = DN(('cn', replica), ('cn', 'masters'), ('cn', 'ipa'),
-                    ('cn', 'etc'), self.suffix)
+            dn = DN(('cn', replica), api.env.container_masters, self.suffix)
             entries = self.conn.get_entries(dn, ldap.SCOPE_SUBTREE)
             if entries:
                 entries.sort(key=lambda x: len(x.dn), reverse=True)
@@ -1768,10 +1835,8 @@ class ReplicationManager(object):
 
     def setup_promote_replication(self, r_hostname, r_binddn=None,
                                   r_bindpw=None, cacert=paths.IPA_CA_CRT):
-        # note - there appears to be a bug in python-ldap - it does not
-        # allow connections using two different CA certs
-        ldap_uri = ipaldap.get_ldap_uri(r_hostname)
-        r_conn = ipaldap.LDAPClient(ldap_uri, cacert=cacert)
+        r_conn = ipaldap.LDAPClient.from_hostname_secure(
+            r_hostname, cacert=cacert)
         if r_bindpw:
             r_conn.simple_bind(r_binddn, r_bindpw)
         else:
@@ -1910,8 +1975,7 @@ class CAReplicationManager(ReplicationManager):
 
     def __init__(self, realm, hostname):
         # Always connect to self over ldapi
-        ldap_uri = ipaldap.get_ldap_uri(protocol='ldapi', realm=realm)
-        conn = ipaldap.LDAPClient(ldap_uri)
+        conn = ipaldap.LDAPClient.from_realm(realm)
         conn.external_bind()
         super(CAReplicationManager, self).__init__(
             realm, hostname, None, port=DEFAULT_PORT, conn=conn)
@@ -1923,8 +1987,7 @@ class CAReplicationManager(ReplicationManager):
         Assumes a promote replica with working GSSAPI for replication
         and unified DS instance.
         """
-        ldap_uri = ipaldap.get_ldap_uri(r_hostname)
-        r_conn = ipaldap.LDAPClient(ldap_uri)
+        r_conn = ipaldap.LDAPClient.from_hostname_secure(r_hostname)
         r_conn.gssapi_bind()
 
         # Setup the first half
