@@ -17,7 +17,23 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
-"""
+
+from copy import deepcopy
+import logging
+
+import six
+
+from ipalib import api, crud, errors
+from ipalib import Object
+from ipalib import Flag, Str, StrEnum, DNParam
+from ipalib.aci import ACI
+from ipalib import output
+from ipalib import _, ngettext
+from ipalib.plugable import Registry
+from .baseldap import gen_pkey_only_option, pkey_to_value
+from ipapython.dn import DN
+
+__doc__ = _("""
 Directory Server Access Control Instructions (ACIs)
 
 ACIs are used to allow or deny access to information. This module is
@@ -115,26 +131,13 @@ The show command shows the raw 389-ds ACI.
 IMPORTANT: When modifying the target attributes of an existing ACI you
 must include all existing attributes as well. When doing an aci-mod the
 targetattr REPLACES the current attributes, it does not add to them.
-
-"""
-from copy import deepcopy
-
-import six
-
-from ipalib import api, crud, errors
-from ipalib import Object
-from ipalib import Flag, Str, StrEnum, DNParam
-from ipalib.aci import ACI
-from ipalib import output
-from ipalib import _, ngettext
-from ipalib.plugable import Registry
-from .baseldap import gen_pkey_only_option, pkey_to_value
-from ipapython.ipa_log_manager import root_logger
-from ipapython.dn import DN
+""")
 
 if six.PY3:
     unicode = str
 
+logger = logging.getLogger(__name__)
+
 register = Registry()
 
 ACI_NAME_PREFIX_SEP = ":"
@@ -280,7 +283,7 @@ def _make_aci(ldap, current, aciname, kw):
             try:
                 api.Object['group'].get_dn_if_exists(kw['memberof'])
             except errors.NotFound:
-                api.Object['group'].handle_not_found(kw['memberof'])
+                raise api.Object['group'].handle_not_found(kw['memberof'])
             groupdn = _group_from_memberof(kw['memberof'])
             a.set_target_filter('memberOf=%s' % groupdn)
         if valid['filter']:
@@ -291,7 +294,7 @@ def _make_aci(ldap, current, aciname, kw):
             if kw['filter'] in ('', None, u''):
                 raise errors.BadSearchFilter(info=_('empty filter'))
             try:
-                entries = ldap.find_entries(filter=kw['filter'])
+                ldap.find_entries(filter=kw['filter'])
             except errors.NotFound:
                 pass
             a.set_target_filter(kw['filter'])
@@ -332,7 +335,7 @@ def _aci_to_kw(ldap, a, test=False, pkey_only=False):
     if 'targetfilter' in a.target:
         target = a.target['targetfilter']['expression']
         if target.startswith('(memberOf=') or target.startswith('memberOf='):
-            (junk, memberof) = target.split('memberOf=', 1)
+            _junk, memberof = target.split('memberOf=', 1)
             memberof = DN(memberof)
             kw['memberof'] = memberof['cn']
         else:
@@ -340,8 +343,8 @@ def _aci_to_kw(ldap, a, test=False, pkey_only=False):
     if 'target' in a.target:
         target = a.target['target']['expression']
         found = False
-        for k in _type_map.keys():
-            if _type_map[k] == target:
+        for k, value in _type_map.items():
+            if value == target:
                 kw['type'] = unicode(k)
                 found = True
                 break
@@ -354,7 +357,8 @@ def _aci_to_kw(ldap, a, test=False, pkey_only=False):
                 try:
                     targetdn = DN(target.replace('ldap:///',''))
                 except ValueError as e:
-                    raise errors.ValidationError(name='subtree', error=_("invalid DN (%s)") % e.message)
+                    raise errors.ValidationError(
+                        name='subtree', error=_("invalid DN (%s)") % e)
                 if targetdn.endswith(DN(api.env.container_group, api.env.basedn)):
                     kw['targetgroup'] = targetdn[0]['cn']
                 else:
@@ -392,8 +396,8 @@ def _convert_strings_to_acis(acistrs):
     for a in acistrs:
         try:
             acis.append(ACI(a))
-        except SyntaxError as e:
-            root_logger.warning("Failed to parse: %s" % a)
+        except SyntaxError:
+            logger.warning("Failed to parse: %s", a)
     return acis
 
 def _find_aci_by_name(acis, aciprefix, aciname):
@@ -408,6 +412,7 @@ def validate_permissions(ugettext, perm):
     perm = perm.strip().lower()
     if perm not in _valid_permissions_values:
         return '"%s" is not a valid permission' % perm
+    return None
 
 
 def _normalize_permissions(perm):
@@ -429,9 +434,7 @@ _prefix_option = StrEnum('aciprefix',
 
 @register()
 class aci(Object):
-    """
-    ACI object.
-    """
+    __doc__ = _('ACI object.')
     NO_CLI = True
 
     label = _('ACIs')
@@ -516,9 +519,7 @@ class aci(Object):
 
 @register()
 class aci_add(crud.Create):
-    """
-    Create new ACI.
-    """
+    __doc__ = _('Create new ACI.')
     NO_CLI = True
     msg_summary = _('Created ACI "%(value)s"')
 
@@ -570,9 +571,7 @@ class aci_add(crud.Create):
 
 @register()
 class aci_del(crud.Delete):
-    """
-    Delete ACI.
-    """
+    __doc__ = _('Delete ACI.')
     NO_CLI = True
     has_output = output.standard_boolean
     msg_summary = _('Deleted ACI "%(value)s"')
@@ -611,9 +610,7 @@ class aci_del(crud.Delete):
 
 @register()
 class aci_mod(crud.Update):
-    """
-    Modify ACI.
-    """
+    __doc__ = _('Modify ACI.')
     NO_CLI = True
 
     takes_options = (_prefix_option,)
@@ -674,8 +671,8 @@ class aci_mod(crud.Update):
 
 @register()
 class aci_find(crud.Search):
-    """
-    Search for ACIs.
+    __doc__ = _("""
+Search for ACIs.
 
     Returns a list of ACIs
 
@@ -692,7 +689,7 @@ class aci_find(crud.Search):
     For example, searching on memberof=ipausers will find all ACIs that
     have ipausers as a memberof. There may be other ACIs that apply to
     members of that group indirectly.
-    """
+    """)
     NO_CLI = True
     msg_summary = ngettext('%(count)d ACI matched', '%(count)d ACIs matched', 0)
 
@@ -788,8 +785,8 @@ class aci_find(crud.Search):
                     results.remove(a)
                     continue
                 found = False
-                for k in _type_map.keys():
-                    if _type_map[k] == target and kw['type'] == k:
+                for k, value in _type_map.items():
+                    if value == target and kw['type'] == k:
                         found = True
                         break
                 if not found:
@@ -880,9 +877,7 @@ class aci_find(crud.Search):
 
 @register()
 class aci_show(crud.Retrieve):
-    """
-    Display a single ACI given an ACI name.
-    """
+    __doc__ = _('Display a single ACI given an ACI name.')
     NO_CLI = True
 
     takes_options = (
@@ -921,9 +916,7 @@ class aci_show(crud.Retrieve):
 
 @register()
 class aci_rename(crud.Update):
-    """
-    Rename an ACI.
-    """
+    __doc__ = _('Rename an ACI.')
     NO_CLI = True
 
     takes_options = (
@@ -944,7 +937,7 @@ class aci_rename(crud.Update):
         aci = _find_aci_by_name(acis, kw['aciprefix'], aciname)
 
         for a in acis:
-            prefix, name = _parse_aci_name(a.name)
+            prefix, _name = _parse_aci_name(a.name)
             if _make_aci_name(prefix, kw['newname']) == a.name:
                 raise errors.DuplicateEntry()
 
