@@ -36,7 +36,7 @@ from ipalib.text import _
 from ipalib.util import json_serialize, validate_hostname
 from ipalib.capabilities import client_has_capability
 from ipalib.messages import add_message, SearchResultTruncated
-from ipapython.dn import DN
+from ipapython.dn import DN, RDN
 from ipapython.version import API_VERSION
 
 if six.PY3:
@@ -200,7 +200,7 @@ def validate_del_attribute(ugettext, attr):
     validate_attribute(ugettext, 'delattr', attr)
 
 def validate_attribute(ugettext, name, attr):
-    m = re.match("\s*(.*?)\s*=\s*(.*?)\s*$", attr)
+    m = re.match(r"\s*(.*?)\s*=\s*(.*?)\s*$", attr)
     if not m or len(m.groups()) != 2:
         raise errors.ValidationError(
             name=name, error=_('Invalid format. Should be name=value'))
@@ -319,6 +319,7 @@ def validate_externalhost(ugettext, hostname):
         validate_hostname(hostname, check_fqdn=False, allow_underscore=True)
     except ValueError as e:
         return unicode(e)
+    return None
 
 
 external_host_param = Str('externalhost*', validate_externalhost,
@@ -375,7 +376,7 @@ def add_external_post_callback(ldap, dn, entry_attrs, failed, completed,
         membertype - the object type of the member (e.g. host)
         externalattr - the attribute name that IPA uses to store the membership
                        of the entries that are not managed by IPA
-                       (e.g externalhost)
+                       (e.g. externalhost)
 
     Returns the number of completed entries so far (the number of entries
     handled by IPA incremented by the number of handled external entries) and
@@ -412,7 +413,7 @@ def add_external_post_callback(ldap, dn, entry_attrs, failed, completed,
             elif (membername in lc_external_entries and
                member_dn not in members):
                 # Already an external member, reset the error message
-                msg = unicode(errors.AlreadyGroupMember().message)
+                msg = unicode(errors.AlreadyGroupMember())
                 newerror = (entry[0], msg)
                 ind = failed[memberattr][membertype].index(entry)
                 failed[memberattr][membertype][ind] = newerror
@@ -445,7 +446,7 @@ def remove_external_post_callback(ldap, dn, entry_attrs, failed, completed,
         membertype - the object type of the member (e.g. host)
         externalattr - the attribute name that IPA uses to store the membership
                        of the entries that are not managed by IPA
-                       (e.g externalhost)
+                       (e.g. externalhost)
 
     Returns the number of completed entries so far (the number of entries
     handled by IPA incremented by the number of handled external entries) and
@@ -472,7 +473,7 @@ def remove_external_post_callback(ldap, dn, entry_attrs, failed, completed,
                     external_entries.remove(entry[0])
                 completed_external += 1
             else:
-                msg = unicode(errors.NotGroupMember().message)
+                msg = unicode(errors.NotGroupMember())
                 newerror = (entry[0], msg)
                 ind = failed[memberattr][membertype].index(entry)
                 failed[memberattr][membertype][ind] = newerror
@@ -496,7 +497,7 @@ def host_is_master(ldap, fqdn):
 
     Raises an exception if a master, otherwise returns nothing.
     """
-    master_dn = DN(('cn', fqdn), ('cn', 'masters'), ('cn', 'ipa'), ('cn', 'etc'), api.env.basedn)
+    master_dn = DN(('cn', fqdn), api.env.container_masters, api.env.basedn)
     try:
         ldap.get_entry(master_dn, ['objectclass'])
         raise errors.ValidationError(name='hostname', error=_('An IPA master host cannot be deleted or disabled'))
@@ -549,7 +550,7 @@ class LDAPObject(Object):
     rdn_attribute = ''
     uuid_attribute = ''
     attribute_members = {}
-    rdn_is_primary_key = False # Do we need RDN change to do a rename?
+    allow_rename = False
     password_attributes = []
     # Can bind as this entry (has userPassword or krbPrincipalKey)
     bindable = False
@@ -563,6 +564,11 @@ class LDAPObject(Object):
         'memberofindirect': (
             'Indirect Member Of', None, 'not_in_indirect_'
         ),
+        'membermanager': (
+            'Group membership managed by',
+            'membermanager_',
+            'not_membermanager_'
+        ),
     }
     label = _('Entry')
     label_singular = _('Entry')
@@ -654,7 +660,7 @@ class LDAPObject(Object):
             del entry_attrs[attr]
 
             for member in value:
-                memberdn = DN(member)
+                memberdn = DN(member.decode('utf-8'))
                 for ldap_obj_name in self.attribute_members[attr]:
                     ldap_obj = self.api.Object[ldap_obj_name]
                     try:
@@ -715,14 +721,16 @@ class LDAPObject(Object):
             result = self.backend.get_entries(
                 self.api.env.basedn,
                 filter=filter,
-                attrs_list=[''])
+                attrs_list=[''],
+                size_limit=-1,  # paged search will get everything anyway
+                paged_search=True)
         except errors.NotFound:
             result = []
 
         direct = set()
         indirect = set(entry.raw.get('memberof', []))
         for group_entry in result:
-            dn = str(group_entry.dn)
+            dn = str(group_entry.dn).encode('utf-8')
             if dn in indirect:
                 indirect.remove(dn)
                 direct.add(dn)
@@ -743,7 +751,7 @@ class LDAPObject(Object):
         for (pwattr, attr) in self.password_attributes:
             search_filter = '(%s=*)' % pwattr
             try:
-                (entries, truncated) = ldap.find_entries(
+                ldap.find_entries(
                     search_filter, [pwattr], dn, ldap.SCOPE_BASE
                 )
                 entry_attrs[attr] = True
@@ -751,6 +759,10 @@ class LDAPObject(Object):
                 entry_attrs[attr] = False
 
     def handle_not_found(self, *keys):
+        """Handle NotFound exception
+
+        Must raise errors.NotFound again.
+        """
         pkey = ''
         if self.primary_key:
             pkey = keys[-1]
@@ -798,14 +810,14 @@ class LDAPObject(Object):
         attrs = self.api.Backend.ldap2.schema.attribute_types(objectclasses)
         attrlist = []
         # Go through the MUST first
-        for (oid, attr) in attrs[0].items():
+        for attr in attrs[0].values():
             attrlist.append(attr.names[0].lower())
         # And now the MAY
-        for (oid, attr) in attrs[1].items():
+        for attr in attrs[1].values():
             attrlist.append(attr.names[0].lower())
         json_dict['aciattrs'] = attrlist
         attrlist.sort()
-        json_dict['methods'] = [m for m in self.methods]
+        json_dict['methods'] = list(self.methods)
         json_dict['can_have_permissions'] = bool(
             self.permission_filter_objectclasses)
         return json_dict
@@ -823,7 +835,7 @@ def _check_single_value_attrs(params, entry_attrs):
 # required, make sure we enforce that.
 def _check_empty_attrs(params, entry_attrs):
     for (a, v) in entry_attrs.items():
-        if v is None or (isinstance(v, six.string_types) and len(v) == 0):
+        if v is None or (isinstance(v, str) and len(v) == 0):
             if a in params and params[a].required:
                 raise errors.RequirementError(name=a)
 
@@ -842,9 +854,13 @@ def _check_limit_object_class(attributes, attrs, allow_only):
     """
     if len(attributes[0]) == 0 and len(attributes[1]) == 0:
         return
-    limitattrs = deepcopy(attrs)
+    # Remove options from the attributes names before validating
+    # LDAP schema does not enforce any of LDAP attribute options
+    # (e.g. attribute;option), thus we should avoid comparing
+    # attribute names with options directly.
+    limitattrs = [x.split(';')[0] for x in attrs]
     # Go through the MUST first
-    for (oid, attr) in attributes[0].items():
+    for attr in attributes[0].values():
         if attr.names[0].lower() in limitattrs:
             if not allow_only:
                 raise errors.ObjectclassViolation(
@@ -852,7 +868,7 @@ def _check_limit_object_class(attributes, attrs, allow_only):
                         attribute=attr.names[0].lower()))
             limitattrs.remove(attr.names[0].lower())
     # And now the MAY
-    for (oid, attr) in attributes[1].items():
+    for attr in attributes[1].values():
         if attr.names[0].lower() in limitattrs:
             if not allow_only:
                 raise errors.ObjectclassViolation(
@@ -909,10 +925,10 @@ last, after all sets and adds."""),
         newdict = {}
         if attrs is None:
             attrs = []
-        elif not type(attrs) in (list, tuple):
+        elif type(attrs) not in (list, tuple):
             attrs = [attrs]
         for a in attrs:
-            m = re.match("\s*(.*?)\s*=\s*(.*?)\s*$", a)
+            m = re.match(r"\s*(.*?)\s*=\s*(.*?)\s*$", a)
             attr = str(m.group(1)).lower()
             value = m.group(2)
             if attr in self.obj.params and attr not in self.params:
@@ -1013,7 +1029,7 @@ last, after all sets and adds."""),
                     dn, needldapattrs
                 )
             except errors.NotFound:
-                self.obj.handle_not_found(*keys)
+                raise self.obj.handle_not_found(*keys)
 
             # Provide a nice error message when user tries to delete an
             # attribute that does not exist on the entry (and user is not
@@ -1036,7 +1052,7 @@ last, after all sets and adds."""),
                     except ValueError:
                         if isinstance(delval, bytes):
                             # This is a Binary value, base64 encode it
-                            delval = unicode(base64.b64encode(delval))
+                            delval = base64.b64encode(delval).decode('ascii')
                         raise errors.AttrValueNotFound(attr=attr, value=delval)
 
         # normalize all values
@@ -1066,7 +1082,9 @@ last, after all sets and adds."""),
                 entry_attrs[attr] = value
             else:
                 # unknown attribute: remove duplicite and invalid values
-                entry_attrs[attr] = list(set([val for val in entry_attrs[attr] if val]))
+                entry_attrs[attr] = list(
+                    {val for val in entry_attrs[attr] if val}
+                )
                 if not entry_attrs[attr]:
                     entry_attrs[attr] = None
                 elif isinstance(entry_attrs[attr], (tuple, list)) and len(entry_attrs[attr]) == 1:
@@ -1142,10 +1160,10 @@ class LDAPCreate(BaseLDAPCommand, crud.Create):
         entry_attrs = ldap.make_entry(
             dn, self.args_options_2_entry(*keys, **options))
 
-        self.process_attr_options(entry_attrs, None, keys, options)
-
         entry_attrs['objectclass'] = deepcopy(self.obj.object_class)
 
+        self.process_attr_options(entry_attrs, None, keys, options)
+
         if self.obj.object_class_config:
             config = ldap.get_ipa_config()
             entry_attrs['objectclass'] = config.get(
@@ -1218,7 +1236,7 @@ class LDAPCreate(BaseLDAPCommand, crud.Create):
                 entry_attrs = self._exc_wrapper(keys, options, ldap.get_entry)(
                     entry_attrs.dn, attrs_list)
         except errors.NotFound:
-            self.obj.handle_not_found(*keys)
+            raise self.obj.handle_not_found(*keys)
 
         self.obj.get_indirect_members(entry_attrs, attrs_list)
 
@@ -1318,7 +1336,7 @@ class LDAPRetrieve(LDAPQuery):
                 dn, attrs_list
             )
         except errors.NotFound:
-            self.obj.handle_not_found(*keys)
+            raise self.obj.handle_not_found(*keys)
 
         self.obj.get_indirect_members(entry_attrs, attrs_list)
 
@@ -1384,7 +1402,7 @@ class LDAPUpdate(LDAPQuery, crud.Update):
     def get_options(self):
         for option in super(LDAPUpdate, self).get_options():
             yield option
-        if self.obj.rdn_is_primary_key:
+        if self.obj.allow_rename:
             yield self._get_rename_option()
 
     def execute(self, *keys, **options):
@@ -1419,15 +1437,19 @@ class LDAPUpdate(LDAPQuery, crud.Update):
         _check_limit_object_class(self.api.Backend.ldap2.schema.attribute_types(self.obj.disallow_object_classes), list(entry_attrs), allow_only=False)
 
         rdnupdate = False
-        try:
-            if self.obj.rdn_is_primary_key and 'rename' in options:
-                if not options['rename']:
-                    raise errors.ValidationError(name='rename', error=u'can\'t be empty')
-                entry_attrs[self.obj.primary_key.name] = options['rename']
+        if 'rename' in options:
+            if not options['rename']:
+                raise errors.ValidationError(
+                    name='rename', error=u'can\'t be empty')
+            entry_attrs[self.obj.primary_key.name] = options['rename']
 
-            if self.obj.rdn_is_primary_key and self.obj.primary_key.name in entry_attrs:
+        # if setattr was used to change the RDN, the primary_key.name is
+        # already in entry_attrs
+        if self.obj.allow_rename and self.obj.primary_key.name in entry_attrs:
+            # perform RDN change if the primary key is also RDN
+            if (RDN((self.obj.primary_key.name, keys[-1])) ==
+                    entry_attrs.dn[0]):
                 try:
-                    # RDN change
                     new_dn = DN((self.obj.primary_key.name,
                                  entry_attrs[self.obj.primary_key.name]),
                                 *entry_attrs.dn[1:])
@@ -1435,17 +1457,21 @@ class LDAPUpdate(LDAPQuery, crud.Update):
                         entry_attrs.dn,
                         new_dn)
 
-                    rdnkeys = keys[:-1] + (entry_attrs[self.obj.primary_key.name], )
+                    rdnkeys = (keys[:-1] +
+                               (entry_attrs[self.obj.primary_key.name], ))
                     entry_attrs.dn = self.obj.get_dn(*rdnkeys)
                     options['rdnupdate'] = True
                     rdnupdate = True
                 except errors.EmptyModlist:
                     # Attempt to rename to the current name, ignore
                     pass
+                except errors.NotFound:
+                    raise self.obj.handle_not_found(*keys)
                 finally:
                     # Delete the primary_key from entry_attrs either way
                     del entry_attrs[self.obj.primary_key.name]
 
+        try:
             # Exception callbacks will need to test for options['rdnupdate']
             # to decide what to do. An EmptyModlist in this context doesn't
             # mean an error occurred, just that there were no other updates to
@@ -1459,7 +1485,7 @@ class LDAPUpdate(LDAPQuery, crud.Update):
             if not rdnupdate:
                 raise e
         except errors.NotFound:
-            self.obj.handle_not_found(*keys)
+            raise self.obj.handle_not_found(*keys)
 
         try:
             entry_attrs = self._exc_wrapper(keys, options, ldap.get_entry)(
@@ -1538,14 +1564,16 @@ class LDAPDelete(LDAPMultiQuery):
                         for entry_attrs in subentries:
                             delete_subtree(entry_attrs.dn)
                 try:
-                    self._exc_wrapper(nkeys, options, ldap.delete_entry)(base_dn)
+                    self._exc_wrapper(nkeys, options, ldap.delete_entry)(
+                        base_dn
+                    )
                 except errors.NotFound:
-                    self.obj.handle_not_found(*nkeys)
+                    raise self.obj.handle_not_found(*nkeys)
 
             try:
                 self._exc_wrapper(nkeys, options, ldap.delete_entry)(dn)
             except errors.NotFound:
-                self.obj.handle_not_found(*nkeys)
+                raise self.obj.handle_not_found(*nkeys)
             except errors.NotAllowedOnNonLeaf:
                 if not self.subtree_delete:
                     raise
@@ -1702,7 +1730,7 @@ class LDAPAddMember(LDAPModMember):
                 dn, attrs_list
             )
         except errors.NotFound:
-            self.obj.handle_not_found(*keys)
+            raise self.obj.handle_not_found(*keys)
 
         self.obj.get_indirect_members(entry_attrs, attrs_list)
 
@@ -1803,7 +1831,7 @@ class LDAPRemoveMember(LDAPModMember):
                 dn, attrs_list
             )
         except errors.NotFound:
-            self.obj.handle_not_found(*keys)
+            raise self.obj.handle_not_found(*keys)
 
         self.obj.get_indirect_members(entry_attrs, attrs_list)
 
@@ -1922,6 +1950,47 @@ class LDAPSearch(BaseLDAPCommand, crud.Search):
             for option in self.get_member_options(attr):
                 yield option
 
+    def get_attr_filter(self, ldap, **options):
+        """
+        Returns a MATCH_ALL filter containing all required attributes from the
+        options
+        """
+        search_kw = self.args_options_2_entry(**options)
+        search_kw['objectclass'] = self.obj.object_class
+
+        filters = []
+        for name, value in search_kw.items():
+            default = self.get_default_of(name, **options)
+            fltr = ldap.make_filter_from_attr(name, value, ldap.MATCH_ALL)
+            if default is not None and value == default:
+                fltr = ldap.combine_filters([fltr, '(!({}=*))'.format(name)])
+            filters.append(fltr)
+
+        return ldap.combine_filters(filters, rules=ldap.MATCH_ALL)
+
+    def get_term_filter(self, ldap, term):
+        """
+        Returns a filter to search for a value (term) in any of the
+        search attributes of an entry.
+        """
+        if self.obj.search_attributes:
+            search_attrs = self.obj.search_attributes
+        else:
+            search_attrs = self.obj.default_attributes
+        if self.obj.search_attributes_config:
+            config = ldap.get_ipa_config()
+            config_attrs = config.get(
+                self.obj.search_attributes_config, [])
+            if len(config_attrs) == 1 and (
+                    isinstance(config_attrs[0], str)):
+                search_attrs = config_attrs[0].split(',')
+
+        search_kw = {}
+        for a in search_attrs:
+            search_kw[a] = term
+
+        return ldap.make_filter(search_kw, exact=False)
+
     def get_member_filter(self, ldap, **options):
         filter = ''
         for attr in self.member_attributes:
@@ -1981,26 +2050,8 @@ class LDAPSearch(BaseLDAPCommand, crud.Search):
                 attrs_list.difference_update(self.obj.attribute_members)
             attrs_list = list(attrs_list)
 
-        if self.obj.search_attributes:
-            search_attrs = self.obj.search_attributes
-        else:
-            search_attrs = self.obj.default_attributes
-        if self.obj.search_attributes_config:
-            config = ldap.get_ipa_config()
-            config_attrs = config.get(
-                self.obj.search_attributes_config, [])
-            if len(config_attrs) == 1 and (
-                isinstance(config_attrs[0], six.string_types)):
-                search_attrs = config_attrs[0].split(',')
-
-        search_kw['objectclass'] = self.obj.object_class
-        attr_filter = ldap.make_filter(search_kw, rules=ldap.MATCH_ALL)
-
-        search_kw = {}
-        for a in search_attrs:
-            search_kw[a] = term
-        term_filter = ldap.make_filter(search_kw, exact=False)
-
+        attr_filter = self.get_attr_filter(ldap, **options)
+        term_filter = self.get_term_filter(ldap, term)
         member_filter = self.get_member_filter(ldap, **options)
 
         filter = ldap.combine_filters(
@@ -2022,10 +2073,13 @@ class LDAPSearch(BaseLDAPCommand, crud.Search):
         except errors.EmptyResult:
             (entries, truncated) = ([], False)
         except errors.NotFound:
-            self.api.Object[self.obj.parent_object].handle_not_found(*keys)
+            return self.api.Object[self.obj.parent_object].handle_not_found(
+                *keys)
 
         for callback in self.get_callbacks('post'):
-            truncated = callback(self, ldap, entries, truncated, *args, **options)
+            truncated = callback(
+                self, ldap, entries, truncated, *args, **options
+            )
 
         if self.sort_result_entries:
             if self.obj.primary_key:
@@ -2035,9 +2089,9 @@ class LDAPSearch(BaseLDAPCommand, crud.Search):
                 entries.sort(key=sort_key)
 
         if not options.get('raw', False):
-            for e in entries:
-                self.obj.get_indirect_members(e, attrs_list)
-                self.obj.convert_attribute_members(e, *args, **options)
+            for entry in entries:
+                self.obj.get_indirect_members(entry, attrs_list)
+                self.obj.convert_attribute_members(entry, *args, **options)
 
         for (i, e) in enumerate(entries):
             entries[i] = entry_to_dict(e, **options)
@@ -2051,9 +2105,9 @@ class LDAPSearch(BaseLDAPCommand, crud.Search):
 
         try:
             ldap.handle_truncated_result(truncated)
-        except errors.LimitsExceeded as e:
+        except errors.LimitsExceeded as exc:
             add_message(options['version'], result, SearchResultTruncated(
-                reason=e))
+                reason=exc))
 
         return result
 
@@ -2300,7 +2354,9 @@ class BaseLDAPModAttribute(LDAPQuery):
         return arg.clone(required=True, attribute=attribute, alwaysask=True)
 
     def _update_attrs(self, update, entry_attrs):
-        raise NotImplementedError("%s.update_attrs()", self.__class__.__name__)
+        raise NotImplementedError(
+            "%s.update_attrs()" % self.__class__.__name__
+        )
 
     def execute(self, *keys, **options):
         ldap = self.obj.backend
@@ -2337,7 +2393,7 @@ class BaseLDAPModAttribute(LDAPQuery):
 
             self._exc_wrapper(keys, options, ldap.update_entry)(update)
         except errors.NotFound:
-            self.obj.handle_not_found(*keys)
+            raise self.obj.handle_not_found(*keys)
 
         try:
             entry_attrs = self._exc_wrapper(keys, options, ldap.get_entry)(
@@ -2375,7 +2431,7 @@ class BaseLDAPModAttribute(LDAPQuery):
 
 
 class BaseLDAPAddAttribute(BaseLDAPModAttribute):
-    msg_summary = _('added attribute value to entry %(value)')
+    msg_summary = _('added attribute value to entry %(value)s')
 
     def _update_attrs(self, update, entry_attrs):
         for name, value in entry_attrs.items():
@@ -2391,7 +2447,7 @@ class BaseLDAPAddAttribute(BaseLDAPModAttribute):
 
 
 class BaseLDAPRemoveAttribute(BaseLDAPModAttribute):
-    msg_summary = _('removed attribute values from entry %(value)')
+    msg_summary = _('removed attribute values from entry %(value)s')
 
     def _update_attrs(self, update, entry_attrs):
         for name, value in entry_attrs.items():
