@@ -137,7 +137,7 @@ class DNOrURL(DNParam):
     """
 
     def _convert_scalar(self, value, index=None):
-        if isinstance(value, six.string_types) and value.startswith('ldap:///'):
+        if isinstance(value, str) and value.startswith('ldap:///'):
             value = strip_ldap_prefix(value)
         return super(DNOrURL, self)._convert_scalar(value)
 
@@ -390,7 +390,7 @@ class permission(baseldap.LDAPObject):
         # memberof
         memberof = []
         for targetfilter in ipapermtargetfilter:
-            match = re.match('^\(memberof=(.*)\)$', targetfilter, re.I)
+            match = re.match(r'^\(memberof=(.*)\)$', targetfilter, re.I)
             if match:
                 try:
                     dn = DN(match.group(1))
