@@ -79,7 +79,7 @@ import six
 
 from ipalib import _, errors
 from ipapython.dn import DN
-
+from ipaserver.masters import ENABLED_SERVICE, HIDDEN_SERVICE
 
 if six.PY3:
     unicode = str
@@ -87,11 +87,12 @@ if six.PY3:
 
 ENABLED = u'enabled'
 CONFIGURED = u'configured'
+HIDDEN = u'hidden'
 ABSENT = u'absent'
 
 
 @six.add_metaclass(abc.ABCMeta)
-class LDAPBasedProperty(object):
+class LDAPBasedProperty:
     """
     base class for all master properties defined by LDAP content
     :param attr_name: attribute name
@@ -103,6 +104,12 @@ class LDAPBasedProperty(object):
     def __init__(self, attr_name, name):
         self.attr_name = attr_name
         self.name = name
+        # for hidden services, insert hidden before '_server' suffix
+        if attr_name.endswith(u'_server'):
+            parts = attr_name.rsplit(u'_', 1)
+            self.attr_name_hidden = u'{}_hidden_server'.format(parts[0])
+        else:
+            self.attr_name_hidden = None
 
 
 @six.add_metaclass(abc.ABCMeta)
@@ -190,6 +197,7 @@ class BaseServerRole(LDAPBasedProperty):
         :returns: * 'enabled' if the role is enabled on the master
                   * 'configured' if it is not enabled but has
                     been configured by installer
+                  * 'hidden' if the role is not advertised
                   * 'absent' otherwise
         """
         ldap2 = api_instance.Backend.ldap2
@@ -336,12 +344,16 @@ class ServerAttribute(LDAPBasedProperty):
         ldap.update_entry(service_entry)
 
     def _get_assoc_role_providers(self, api_instance):
-        """
-        get list of all servers on which the associated role is enabled
+        """get list of all servers on which the associated role is enabled
+
+        Consider a hidden and configured server as a valid provider for a
+        role, as all services are started.
         """
         return [
-            r[u'server_server'] for r in self.associated_role.status(
-                api_instance) if r[u'status'] == ENABLED]
+            r[u'server_server']
+            for r in self.associated_role.status(api_instance)
+            if r[u'status'] in {ENABLED, HIDDEN, CONFIGURED}
+        ]
 
     def _remove(self, api_instance, masters):
         """
@@ -442,7 +454,7 @@ class SingleValuedServerAttribute(ServerAttribute):
         return masters
 
 
-_Service = namedtuple('Service', ['name', 'enabled'])
+_Service = namedtuple('Service', ['name', 'enabled', 'hidden'])
 
 
 class ServiceBasedRole(BaseServerRole):
@@ -470,8 +482,9 @@ class ServiceBasedRole(BaseServerRole):
         entry_cn = entry['cn'][0]
 
         enabled = self._is_service_enabled(entry)
+        hidden = self._is_service_hidden(entry)
 
-        return _Service(name=entry_cn, enabled=enabled)
+        return _Service(name=entry_cn, enabled=enabled, hidden=hidden)
 
     def _is_service_enabled(self, entry):
         """
@@ -483,11 +496,17 @@ class ServiceBasedRole(BaseServerRole):
         :param entry: LDAPEntry of the service
         :returns: True if the service entry is enabled, False otherwise
         """
-        enabled_value = 'enabledservice'
-        ipaconfigstring_values = set(
-            e.lower() for e in entry.get('ipaConfigString', []))
+        ipaconfigstring_values = set(entry.get('ipaConfigString', []))
+        return ENABLED_SERVICE in ipaconfigstring_values
 
-        return enabled_value in ipaconfigstring_values
+    def _is_service_hidden(self, entry):
+        """Determine if service is hidden
+
+        :param entry: LDAPEntry of the service
+        :returns: True if the service entry is enabled, False otherwise
+        """
+        ipaconfigstring_values = set(entry.get('ipaConfigString', []))
+        return HIDDEN_SERVICE in ipaconfigstring_values
 
     def _get_services_by_masters(self, entries):
         """
@@ -512,9 +531,12 @@ class ServiceBasedRole(BaseServerRole):
             except ValueError:
                 continue
 
-            status = (
-                ENABLED if all(s.enabled for s in services) else
-                CONFIGURED)
+            if all(s.enabled for s in services):
+                status = ENABLED
+            elif all(s.hidden for s in services):
+                status = HIDDEN
+            else:
+                status = CONFIGURED
 
             result.append(self.create_role_status_dict(master, status))
 
