@@ -28,6 +28,7 @@ import pytest
 from ipaplatform.constants import constants
 from ipaplatform.paths import paths
 from ipaplatform.tasks import tasks as platformtasks
+from ipapython.ipaldap import realm_to_serverid
 from ipapython.dn import DN
 from ipapython import ipautil
 from ipatests.test_integration.base import IntegrationTest
@@ -76,7 +77,7 @@ def check_admin_in_cli(host):
     # LDAP do not guarantee any order, so the test cannot assume it. Based on
     # that, the code bellow order the 'Member of groups' field to able to
     # assert it latter.
-    data = dict(re.findall("\W*(.+):\W*(.+)\W*", result.stdout_text))
+    data = dict(re.findall(r"\W*(.+):\W*(.+)\W*", result.stdout_text))
     data["Member of groups"] = ', '.join(sorted(data["Member of groups"]
                                                 .split(", ")))
     result.stdout_text = ''.join([' {}: {}\n'.format(k, v)
@@ -115,6 +116,17 @@ def check_custodia_files(host):
     return True
 
 
+def check_pkcs11_modules(host):
+    """regression test for https://pagure.io/freeipa/issue/8073"""
+    # Return a dictionary with key = filename, value = file content
+    # containing all the PKCS11 modules modified by the installer
+    result = dict()
+    for filename in platformtasks.get_pkcs11_modules():
+        assert host.transport.file_exists(filename)
+        result[filename] = host.get_file_contents(filename)
+    return result
+
+
 CHECKS = [
     (check_admin_in_ldap, assert_entries_equal),
     (check_admin_in_cli, assert_results_equal),
@@ -122,7 +134,8 @@ CHECKS = [
     (check_certs, assert_results_equal),
     (check_dns, assert_results_equal),
     (check_kinit, assert_results_equal),
-    (check_custodia_files, assert_deepequal)
+    (check_custodia_files, assert_deepequal),
+    (check_pkcs11_modules, assert_deepequal)
 ]
 
 
@@ -138,6 +151,8 @@ def restore_checker(host):
 
     yield
 
+    # Wait for SSSD to become online before doing any other check
+    tasks.wait_for_sssd_domain_status_online(host)
     tasks.kinit_admin(host)
 
     for (check, assert_func), expected in zip(CHECKS, results):
@@ -152,7 +167,7 @@ def backup(host):
 
     # Test for ticket 7632: check that services are restarted
     # before the backup is compressed
-    pattern = r'.*gzip.*Starting IPA service.*'
+    pattern = r'.*{}.*Starting IPA service.*'.format(paths.GZIP)
     if (re.match(pattern, result.stderr_text, re.DOTALL)):
         raise AssertionError('IPA Services are started after compression')
 
@@ -205,6 +220,15 @@ class TestBackupAndRestore(IntegrationTest):
             self.master.run_command(['ipa-restore', backup_path],
                                     stdin_text=dirman_password + '\nyes')
 
+            # check the file permssion and ownership is set to 770 and
+            # dirsrv:dirsrv after restore on /var/log/dirsrv/slapd-<instance>
+            # related ticket : https://pagure.io/freeipa/issue/7725
+            instance = realm_to_serverid(self.master.domain.realm)
+            log_path = paths.VAR_LOG_DIRSRV_INSTANCE_TEMPLATE % instance
+            cmd = self.master.run_command(['stat', '-c',
+                                           '"%a %G:%U"', log_path])
+            assert "770 dirsrv:dirsrv" in cmd.stdout_text
+
     def test_full_backup_and_restore_with_removed_users(self):
         """regression test for https://fedorahosted.org/freeipa/ticket/3866"""
         with restore_checker(self.master):
@@ -275,7 +299,10 @@ class BaseBackupAndRestoreWithDNS(IntegrationTest):
         tasks.install_master(cls.master, setup_dns=True)
 
     def _full_backup_restore_with_DNS_zone(self, reinstall=False):
-        """backup, uninstall, restore"""
+        """backup, uninstall, restore.
+
+        Test for bug https://pagure.io/freeipa/issue/7630.
+        """
         with restore_checker(self.master):
             self.master.run_command([
                 'ipa', 'dnszone-add',
@@ -290,13 +317,20 @@ class BaseBackupAndRestoreWithDNS(IntegrationTest):
                                      '--uninstall',
                                      '-U'])
 
-            if reinstall:
-                tasks.install_master(self.master, setup_dns=True)
+            tasks.uninstall_packages(self.master, ['*ipa-server-dns'])
 
             dirman_password = self.master.config.dirman_password
+            result = self.master.run_command(
+                ['ipa-restore', backup_path],
+                stdin_text=dirman_password + '\nyes',
+                raiseonerr=False)
+            assert 'Please install the package' in result.stderr_text
+
+            tasks.install_packages(self.master, ['*ipa-server-dns'])
+            if reinstall:
+                tasks.install_master(self.master, setup_dns=True)
             self.master.run_command(['ipa-restore', backup_path],
                                     stdin_text=dirman_password + '\nyes')
-
             tasks.resolve_record(self.master.ip, self.example_test_zone)
 
             tasks.kinit_admin(self.master)
@@ -839,3 +873,39 @@ class TestReplicaInstallAfterRestore(IntegrationTest):
         # install second replica after restore
         tasks.install_replica(master, replica2)
         check_replication(master, replica2, "testuser2")
+
+
+class TestBackupAndRestoreTrust(IntegrationTest):
+    """Test for Backup and Restore for Trust scenario"""
+    topology = 'star'
+
+    def test_restore_trust_pkg_before_restore(self):
+        """Test restore with adtrust when trust-ad pkg is missing.
+
+        Test for bug https://pagure.io/freeipa/issue/7630.
+        """
+        tasks.install_packages(self.master, ['*ipa-server-trust-ad'])
+        self.master.run_command(['ipa-adtrust-install', '-U',
+                                 '--enable-compat', '--netbios-name', 'IPA',
+                                 '-a', self.master.config.admin_password,
+                                 '--add-sids'])
+
+        with restore_checker(self.master):
+            backup_path = backup(self.master)
+
+            self.master.run_command(['ipa-server-install',
+                                     '--uninstall',
+                                     '-U'])
+
+            tasks.uninstall_packages(self.master, ['*ipa-server-trust-ad'])
+
+            dirman_password = self.master.config.dirman_password
+            result = self.master.run_command(
+                ['ipa-restore', backup_path],
+                stdin_text=dirman_password + '\nyes',
+                raiseonerr=False)
+            assert 'Please install the package' in result.stderr_text
+
+            tasks.install_packages(self.master, ['*ipa-server-trust-ad'])
+            self.master.run_command(['ipa-restore', backup_path],
+                                    stdin_text=dirman_password + '\nyes')
