@@ -3,17 +3,20 @@
 from __future__ import absolute_import
 
 import re
-import unittest
 import textwrap
 
+import pytest
+
 from ipaplatform.constants import constants as platformconstants
 from ipaplatform.paths import paths
 
 from ipatests.test_integration.base import IntegrationTest
 from ipatests.pytest_ipa.integration import tasks
+from ipapython.dn import DN
 
 
 class BaseTestTrust(IntegrationTest):
+    num_clients = 1
     topology = 'line'
     num_ad_domains = 1
     num_ad_subdomains = 1
@@ -31,17 +34,18 @@ class BaseTestTrust(IntegrationTest):
     @classmethod
     def install(cls, mh):
         if not cls.master.transport.file_exists('/usr/bin/rpcclient'):
-            raise unittest.SkipTest("Package samba-client not available "
-                                    "on {}".format(cls.master.hostname))
+            raise pytest.skip("Package samba-client not available "
+                              "on {}".format(cls.master.hostname))
         super(BaseTestTrust, cls).install(mh)
-        cls.ad = cls.ads[0]  # pylint: disable=no-member
+        cls.ad = cls.ads[0]
         cls.ad_domain = cls.ad.domain.name
         tasks.install_adtrust(cls.master)
         cls.check_sid_generation()
+        tasks.sync_time(cls.master, cls.ad)
 
-        cls.child_ad = cls.ad_subdomains[0]  # pylint: disable=no-member
+        cls.child_ad = cls.ad_subdomains[0]
         cls.ad_subdomain = cls.child_ad.domain.name
-        cls.tree_ad = cls.ad_treedomains[0]  # pylint: disable=no-member
+        cls.tree_ad = cls.ad_treedomains[0]
         cls.ad_treedomain = cls.tree_ad.domain.name
 
         # values used in workaround for
@@ -63,10 +67,6 @@ class BaseTestTrust(IntegrationTest):
         tasks.run_repeatedly(cls.master, command,
                              test=lambda x: re.search(stdout_re, x))
 
-    def configure_dns_and_time(self, ad_host):
-        tasks.configure_dns_for_trust(self.master, ad_host)
-        tasks.sync_time(self.master, ad_host)
-
     def check_trustdomains(self, realm, expected_ad_domains):
         """Check that ipa trustdomain-find lists all expected domains"""
         result = self.master.run_command(['ipa', 'trustdomain-find', realm])
@@ -89,7 +89,6 @@ class BaseTestTrust(IntegrationTest):
 
     def remove_trust(self, ad):
         tasks.remove_trust_with_ad(self.master, ad.domain.name)
-        tasks.unconfigure_dns_for_trust(self.master, ad)
         tasks.clear_sssd_cache(self.master)
 
 
@@ -98,7 +97,7 @@ class TestTrust(BaseTestTrust):
     # Tests for non-posix AD trust
 
     def test_establish_nonposix_trust(self):
-        self.configure_dns_and_time(self.ad)
+        tasks.configure_dns_for_trust(self.master, self.ad)
         tasks.establish_trust_with_ad(
             self.master, self.ad_domain,
             extra_args=['--range-type', 'ipa-ad-trust'])
@@ -179,11 +178,12 @@ class TestTrust(BaseTestTrust):
 
     def test_remove_nonposix_trust(self):
         self.remove_trust(self.ad)
+        tasks.unconfigure_dns_for_trust(self.master, self.ad)
 
     # Tests for posix AD trust
 
     def test_establish_posix_trust(self):
-        self.configure_dns_and_time(self.ad)
+        tasks.configure_dns_for_trust(self.master, self.ad)
         tasks.establish_trust_with_ad(
             self.master, self.ad_domain,
             extra_args=['--range-type', 'ipa-ad-trust-posix'])
@@ -243,14 +243,9 @@ class TestTrust(BaseTestTrust):
 
         try:
             testuser = 'testuser@%s' % self.ad_domain
-            domain = self.master.domain
-            tasks.modify_sssd_conf(
-                self.master,
-                domain.name,
-                {
-                    'subdomain_homedir': '%o'
-                }
-            )
+            with tasks.remote_sssd_config(self.master) as sssd_conf:
+                sssd_conf.edit_domain(self.master.domain,
+                                      'subdomain_homedir', '%o')
 
             tasks.clear_sssd_cache(self.master)
             # The initgroups operation now uses the LDAP connection because
@@ -268,8 +263,67 @@ class TestTrust(BaseTestTrust):
             tasks.restore_files(self.master)
             tasks.clear_sssd_cache(self.master)
 
+    def test_extdom_plugin(self):
+        """Extdom plugin should not return error (32)/'No such object'
+
+        Regression test for https://pagure.io/freeipa/issue/8044
+
+        If there is a timeout during a request to SSSD the extdom plugin
+        should not return error 'No such object' and the existing user should
+        not be added to negative cache on the client.
+        """
+        extdom_dn = DN(
+            ('cn', 'ipa_extdom_extop'), ('cn', 'plugins'),
+            ('cn', 'config')
+        )
+
+        client = self.clients[0]
+        tasks.backup_file(self.master, paths.SSSD_CONF)
+        log_file = '{0}/sssd_{1}.log'.format(paths.VAR_LOG_SSSD_DIR,
+                                             client.domain.name)
+        logsize = len(client.get_file_contents(log_file))
+        res = self.master.run_command(['pidof', 'sssd_be'])
+        pid = res.stdout_text.strip()
+        test_id = 'id testuser@%s' % self.ad_domain
+        client.run_command(test_id)
+
+        conn = self.master.ldap_connect()
+        entry = conn.get_entry(extdom_dn)  # pylint: disable=no-member
+        orig_extdom_timeout = entry.single_value.get('ipaextdommaxnsstimeout')
+
+        # set the extdom plugin timeout to 1s (1000)
+        entry.single_value['ipaextdommaxnsstimeout'] = 1000
+        conn.update_entry(entry)  # pylint: disable=no-member
+        self.master.run_command(['ipactl', 'restart'])
+
+        with tasks.remote_sssd_config(self.master) as sssd_conf:
+            sssd_conf.edit_domain(self.master.domain, 'timeout', '999999')
+
+        remove_cache = 'sss_cache -E'
+        self.master.run_command(remove_cache)
+        client.run_command(remove_cache)
+
+        try:
+            # stop sssd_be, needed to simulate a timeout in the extdom plugin.
+            stop_sssdbe = self.master.run_command('kill -STOP %s' % pid)
+            client.run_command(test_id)
+            error = 'ldap_extended_operation result: No such object(32)'
+            sssd_log2 = client.get_file_contents(log_file)[logsize:]
+            assert error.encode() not in sssd_log2
+        finally:
+            if stop_sssdbe.returncode == 0:
+                self.master.run_command('kill -CONT %s' % pid)
+            # reconnect and set back to default extdom plugin
+            conn = self.master.ldap_connect()
+            entry = conn.get_entry(extdom_dn)  # pylint: disable=no-member
+            entry.single_value['ipaextdommaxnsstimeout'] = orig_extdom_timeout
+            conn.update_entry(entry)  # pylint: disable=no-member
+            tasks.restore_files(self.master)
+            self.master.run_command(['ipactl', 'restart'])
+
     def test_remove_posix_trust(self):
         self.remove_trust(self.ad)
+        tasks.unconfigure_dns_for_trust(self.master, self.ad)
 
     # Tests for handling invalid trust types
 
@@ -281,7 +335,7 @@ class TestTrust(BaseTestTrust):
                                'random-invalid',
                                're@ll%ybad12!']
 
-        self.configure_dns_and_time(self.ad)
+        tasks.configure_dns_for_trust(self.master, self.ad)
         try:
             for range_type in invalid_range_types:
                 tasks.kinit_admin(self.master)
@@ -302,7 +356,7 @@ class TestTrust(BaseTestTrust):
     # Tests for external trust with AD subdomain
 
     def test_establish_external_subdomain_trust(self):
-        self.configure_dns_and_time(self.child_ad)
+        tasks.configure_dns_for_trust(self.master, self.ad)
         tasks.establish_trust_with_ad(
             self.master, self.ad_subdomain,
             extra_args=['--range-type', 'ipa-ad-trust', '--external=True'])
@@ -328,11 +382,12 @@ class TestTrust(BaseTestTrust):
 
     def test_remove_external_subdomain_trust(self):
         self.remove_trust(self.child_ad)
+        tasks.unconfigure_dns_for_trust(self.master, self.ad)
 
     # Tests for non-external trust with AD subdomain
 
     def test_establish_nonexternal_subdomain_trust(self):
-        self.configure_dns_and_time(self.child_ad)
+        tasks.configure_dns_for_trust(self.master, self.ad)
         try:
             tasks.kinit_admin(self.master)
 
@@ -347,12 +402,12 @@ class TestTrust(BaseTestTrust):
             assert ("Domain '{0}' is not a root domain".format(
                 self.ad_subdomain) in result.stderr_text)
         finally:
-            tasks.unconfigure_dns_for_trust(self.master, self.child_ad)
+            tasks.unconfigure_dns_for_trust(self.master, self.ad)
 
     # Tests for external trust with tree domain
 
     def test_establish_external_treedomain_trust(self):
-        self.configure_dns_and_time(self.tree_ad)
+        tasks.configure_dns_for_trust(self.master, self.ad, self.tree_ad)
         tasks.establish_trust_with_ad(
             self.master, self.ad_treedomain,
             extra_args=['--range-type', 'ipa-ad-trust', '--external=True'])
@@ -379,11 +434,12 @@ class TestTrust(BaseTestTrust):
 
     def test_remove_external_treedomain_trust(self):
         self.remove_trust(self.tree_ad)
+        tasks.unconfigure_dns_for_trust(self.master, self.ad, self.tree_ad)
 
     # Test for non-external trust with tree domain
 
     def test_establish_nonexternal_treedomain_trust(self):
-        self.configure_dns_and_time(self.tree_ad)
+        tasks.configure_dns_for_trust(self.master, self.ad, self.tree_ad)
         try:
             tasks.kinit_admin(self.master)
 
@@ -398,12 +454,12 @@ class TestTrust(BaseTestTrust):
             assert ("Domain '{0}' is not a root domain".format(
                 self.ad_treedomain) in result.stderr_text)
         finally:
-            tasks.unconfigure_dns_for_trust(self.master, self.tree_ad)
+            tasks.unconfigure_dns_for_trust(self.master, self.ad, self.tree_ad)
 
     # Tests for external trust with root domain
 
     def test_establish_external_rootdomain_trust(self):
-        self.configure_dns_and_time(self.ad)
+        tasks.configure_dns_for_trust(self.master, self.ad)
         tasks.establish_trust_with_ad(
             self.master, self.ad_domain,
             extra_args=['--range-type', 'ipa-ad-trust', '--external=True'])
@@ -413,11 +469,12 @@ class TestTrust(BaseTestTrust):
 
     def test_remove_external_rootdomain_trust(self):
         self.remove_trust(self.ad)
+        tasks.unconfigure_dns_for_trust(self.master, self.ad)
 
     # Test for one-way forest trust with shared secret
 
     def test_establish_forest_trust_with_shared_secret(self):
-        self.configure_dns_and_time(self.ad)
+        tasks.configure_dns_for_trust(self.master, self.ad)
         tasks.configure_windows_dns_for_trust(self.ad, self.master)
 
         # this is a workaround for
@@ -484,11 +541,12 @@ class TestTrust(BaseTestTrust):
              self.srv_gc_record_value])
 
         tasks.unconfigure_windows_dns_for_trust(self.ad, self.master)
+        tasks.unconfigure_dns_for_trust(self.master, self.ad)
 
     # Test for one-way external trust with shared secret
 
     def test_establish_external_trust_with_shared_secret(self):
-        self.configure_dns_and_time(self.ad)
+        tasks.configure_dns_for_trust(self.master, self.ad)
         tasks.configure_windows_dns_for_trust(self.ad, self.master)
 
         # create windows side of trust using netdom.exe utility
@@ -535,6 +593,7 @@ class TestTrust(BaseTestTrust):
         )
         self.remove_trust(self.ad)
         tasks.unconfigure_windows_dns_for_trust(self.ad, self.master)
+        tasks.unconfigure_dns_for_trust(self.master, self.ad)
 
     def test_server_option_with_unreachable_ad(self):
         """
@@ -564,7 +623,7 @@ class TestTrust(BaseTestTrust):
             _ldap._tcp IN SRV 0 100 389 unreachable.{ad_dom}.
             _kerberos._udp IN SRV 0 100 88 unreachable.{ad_dom}.
             _kpasswd._udp IN SRV 0 100 464 unreachable.{ad_dom}.
-            ad1 IN A {ad_ip}
+            {ad_short} IN A {ad_ip}
             unreachable IN A {unreachable}
             DomainDnsZones IN A {ad_ip}
             _ldap._tcp.Default-First-Site-Name._sites.DomainDnsZones IN SRV 0 100 389 unreachable.{ad_dom}.
@@ -574,7 +633,8 @@ class TestTrust(BaseTestTrust):
             _ldap._tcp.ForestDnsZones IN SRV 0 100 389 unreachable.{ad_dom}.
         '''.format(  # noqa: E501
             ad_ip=self.ad.ip, unreachable='192.168.254.254',
-            ad_host=self.ad.hostname, ad_dom=self.ad.domain.name))
+            ad_host=self.ad.hostname, ad_dom=self.ad.domain.name,
+            ad_short=self.ad.shortname))
         ad_zone_file = tasks.create_temp_file(self.master, directory='/etc')
         self.master.put_file_contents(ad_zone_file, ad_zone)
         self.master.run_command(
@@ -596,7 +656,8 @@ class TestTrust(BaseTestTrust):
             # This checks that our setup is correct
             result = self.master.run_command(
                 ['ipa', 'trust-add', self.ad.domain.name,
-                 '--admin', 'Administrator', '--password'], raiseonerr=False)
+                 '--admin', 'Administrator', '--password'], raiseonerr=False,
+                stdin_text=self.master.config.ad_admin_password)
             assert result.returncode == 1
             assert 'CIFS server communication error: code "3221225653", ' \
                    'message "{Device Timeout}' in result.stderr_text
@@ -624,7 +685,7 @@ class TestTrust(BaseTestTrust):
             assert ('List of trust domains successfully refreshed'
                     in result.stdout_text)
         finally:
+            self.remove_trust(self.ad)
             tasks.restore_files(self.master)
             self.master.run_command(['rm', '-f', ad_zone_file])
             tasks.restart_named(self.master)
-            tasks.remove_trust_with_ad(self.master, self.ad_domain)
