@@ -28,6 +28,7 @@ import pwd
 import socket
 import sys
 import tempfile
+import textwrap
 
 import pytest
 import six
@@ -84,8 +85,9 @@ def test_ip_address(addr, words, prefixlen):
         assert ip.prefixlen == prefixlen
 
 
-class TestCIDict(object):
-    def setup(self):
+class TestCIDict:
+    @pytest.fixture(autouse=True)
+    def cidict_setup(self):
         self.cidict = ipautil.CIDict()
         self.cidict["Key1"] = "val1"
         self.cidict["key2"] = "val2"
@@ -337,7 +339,7 @@ class TestCIDict(object):
         assert list(dct.values()) == [None] * 3
 
 
-class TestTimeParser(object):
+class TestTimeParser:
     def test_simple(self):
         timestr = "20070803"
 
@@ -393,8 +395,6 @@ class TestTimeParser(object):
         assert_equal(800000, time.microsecond)
 
     def test_time_zones(self):
-        # pylint: disable=no-member
-
         timestr = "20051213141205Z"
 
         time = ipautil.parse_generalized_time(timestr)
@@ -577,3 +577,32 @@ def test_check_port_bindable_udp(udp_listen):
     assert not ipautil.check_port_bindable(port, socket.SOCK_DGRAM)
     sock.close()
     assert ipautil.check_port_bindable(port, socket.SOCK_DGRAM)
+
+
+def test_config_replace_variables(tempdir):
+    conffile = os.path.join(tempdir, 'test.conf')
+
+    conf = textwrap.dedent("""
+    replaced=foo
+    removed=gone
+    """)
+    expected = textwrap.dedent("""
+    replaced=bar
+    addreplaced=baz
+    """)
+
+    with open(conffile, 'w') as f:
+        f.write(conf)
+
+    result = ipautil.config_replace_variables(
+        conffile,
+        replacevars=dict(replaced="bar", addreplaced="baz"),
+        removevars={'removed'}
+    )
+    assert result == {
+        'removed': 'gone', 'replaced': 'foo'
+    }
+
+    with open(conffile, 'r') as f:
+        newconf = f.read()
+    assert newconf == expected
