@@ -20,25 +20,21 @@
 Base class for HTTP request tests
 """
 
-from six.moves import urllib
+import urllib
 
-from ipalib import api
+from ipalib import api, util
 
-# Python 3 rename. The package is available in "six.moves.http_client", but
-# pylint cannot handle classes from that alias
-try:
-    import httplib
-except ImportError:
-    import http.client as httplib
 
-class Unauthorized_HTTP_test(object):
+class Unauthorized_HTTP_test:
     """
     Base class for simple HTTP request tests executed against URI
     with no required authorization
     """
     app_uri = ''
     host = api.env.host
+    cacert = api.env.tls_ca_cert
     content_type = 'application/x-www-form-urlencoded'
+    accept_language = 'en-us'
 
     def send_request(self, method='POST', params=None):
         """
@@ -47,14 +43,17 @@ class Unauthorized_HTTP_test(object):
         :param key When not None, overrides default app_uri
         """
         if params is not None:
-            # urlencode *can* take two arguments
-            # pylint: disable=too-many-function-args
-            params = urllib.parse.urlencode(params, True)
+            if self.content_type == 'application/x-www-form-urlencoded':
+                # urlencode *can* take two arguments
+                # pylint: disable=too-many-function-args
+                params = urllib.parse.urlencode(params, True)
         url = 'https://' + self.host + self.app_uri
 
-        headers = {'Content-Type' : self.content_type,
-                   'Referer' : url}
+        headers = {'Content-Type': self.content_type,
+                   'Accept-Language': self.accept_language,
+                   'Referer': url}
 
-        conn = httplib.HTTPSConnection(self.host)
+        conn = util.create_https_connection(
+                self.host, cafile=self.cacert)
         conn.request(method, self.app_uri, params, headers)
         return conn.getresponse()
