@@ -28,25 +28,27 @@ from datetime import datetime
 import time
 import re
 import os
-from functools import wraps
-import unittest
-import paramiko
 
-# pylint: disable=import-error
-from six.moves.urllib.error import URLError
-# pylint: enable=import-error
+from functools import wraps
+from urllib.error import URLError
+
+import paramiko
+import pytest
 
 try:
     from selenium import webdriver
     from selenium.common.exceptions import NoSuchElementException
     from selenium.common.exceptions import InvalidElementStateException
     from selenium.common.exceptions import StaleElementReferenceException
+    from selenium.common.exceptions import UnexpectedAlertPresentException
     from selenium.common.exceptions import WebDriverException
     from selenium.common.exceptions import ElementClickInterceptedException
+    from selenium.webdriver.common.action_chains import ActionChains
     from selenium.webdriver.common.desired_capabilities import DesiredCapabilities
     from selenium.webdriver.common.keys import Keys
     from selenium.webdriver.common.by import By
     from selenium.webdriver.chrome.options import Options as ChromeOptions
+    from selenium.webdriver.support.expected_conditions import alert_is_present
     from selenium.webdriver.support.wait import WebDriverWait
     from selenium.webdriver.support.ui import Select
     NO_SELENIUM = False
@@ -57,7 +59,9 @@ try:
     NO_YAML = False
 except ImportError:
     NO_YAML = True
+
 from ipaplatform.paths import paths
+from ipaplatform.tasks import tasks
 
 ENV_MAP = {
     'MASTER': 'ipa_server',
@@ -95,11 +99,9 @@ def screenshot(fn):
     Should be applied on methods of UI_driver subclasses
     """
     @wraps(fn)
-    def screenshot_wrapper(*args):
+    def screenshot_wrapper(*args, **kwargs):
         try:
-            return fn(*args)
-        except unittest.SkipTest:
-            raise
+            return fn(*args, **kwargs)
         except Exception:
             self = args[0]
             name = '%s_%s_%s' % (
@@ -112,26 +114,68 @@ def screenshot(fn):
     return screenshot_wrapper
 
 
-class UI_driver(object):
+def dismiss_unexpected_alert(fn):
+    """
+    Temporary fix for UnexpectedAlertPresentException.
+    It is regression in Firefox 55
+    Fixed in Firefox 65:
+    https://bugzilla.mozilla.org/show_bug.cgi?id=1503015
+    """
+    @wraps(fn)
+    def wrapped(*args, **kwargs):
+        self = args[0]
+        try:
+            return fn(*args, **kwargs)
+        except UnexpectedAlertPresentException:
+            if alert_is_present()(self.driver):
+                self.driver.switch_to.alert.dismiss()
+            # One retry is enough for now.
+            # But in the case of catching two alerts at the same time
+            # loop or recursive call should be used.
+            return fn(*args, **kwargs)
+    return wrapped
+
+
+def repeat_on_stale_parent_reference(fn):
+    """
+    The decorator repeats a function once when StaleElementReferenceException
+    is caught.
+    It is not applicable if a parent reference is created outside a function.
+    """
+    @wraps(fn)
+    def wrapped(*args, **kwargs):
+        if ('parent' in kwargs) and kwargs['parent'] is None:
+            try:
+                return fn(*args, **kwargs)
+            except StaleElementReferenceException:
+                pass
+        return fn(*args, **kwargs)
+    return wrapped
+
+
+class UI_driver:
     """
     Base class for all UI integration tests
     """
 
     request_timeout = 60
 
-    @classmethod
-    def setup_class(cls):
+    @pytest.fixture(autouse=True, scope="class")
+    def ui_driver_setup(self, request):
+        cls = request.cls
         if NO_SELENIUM:
-            raise unittest.SkipTest('Selenium not installed')
+            pytest.skip('Selenium not installed')
         cls.load_config()
 
-    def setup(self):
+    @pytest.fixture(autouse=True)
+    def ui_driver_fsetup(self, request):
         self.driver = self.get_driver()
         self.driver.maximize_window()
 
-    def teardown(self):
-        self.driver.delete_all_cookies()
-        self.driver.quit()
+        def fin():
+            self.driver.delete_all_cookies()
+            self.driver.quit()
+        request.addfinalizer(fin)
 
     @classmethod
     def load_config(cls):
@@ -149,9 +193,9 @@ class UI_driver(object):
                 with open(path, 'r') as conf:
                     cls.config = yaml.load(conf)
             except yaml.YAMLError as e:
-                raise unittest.SkipTest("Invalid Web UI config.\n%s" % e)
+                pytest.skip("Invalid Web UI config.\n%s" % e)
             except IOError as e:
-                raise unittest.SkipTest(
+                pytest.skip(
                     "Can't load Web UI test config: %s" % e
                 )
         else:
@@ -190,7 +234,7 @@ class UI_driver(object):
 
         if driver_type == 'remote':
             if 'host' not in cls.config:
-                raise unittest.SkipTest('Selenium server host not configured')
+                pytest.skip('Selenium server host not configured')
             host = cls.config["host"]
 
             if browser == 'chrome':
@@ -206,11 +250,11 @@ class UI_driver(object):
                     command_executor='http://%s:%d/wd/hub' % (host, port),
                     desired_capabilities=capabilities)
             except URLError as e:
-                raise unittest.SkipTest(
+                pytest.skip(
                     'Error connecting to selenium server: %s' % e
                 )
             except RuntimeError as e:
-                raise unittest.SkipTest(
+                pytest.skip(
                     'Error while establishing webdriver: %s' % e
                 )
         else:
@@ -226,16 +270,17 @@ class UI_driver(object):
                     ff_log_path = cls.config.get("geckodriver_log_path")
                     driver = webdriver.Firefox(fp, log_path=ff_log_path)
             except URLError as e:
-                raise unittest.SkipTest(
+                pytest.skip(
                     'Error connecting to selenium server: %s' % e
                 )
             except RuntimeError as e:
-                raise unittest.SkipTest(
+                pytest.skip(
                     'Error while establishing webdriver: %s' % e
                 )
 
         return driver
 
+    @dismiss_unexpected_alert
     def find(self, expression, by='id', context=None, many=False, strict=False):
         """
         Helper which calls selenium find_element_by_xxx methods.
@@ -270,6 +315,10 @@ class UI_driver(object):
 
         return result
 
+    def find_by_selector(self, expression, context=None, **kwargs):
+        return self.find(expression, by=By.CSS_SELECTOR, context=context,
+                         **kwargs)
+
     def files_loaded(self):
         """
         Test if dependencies were loaded. (Checks if UI has been rendered)
@@ -333,6 +382,19 @@ class UI_driver(object):
             self.wait()
         self.wait(d)
 
+    def wait_while_working(self, widget, implicit=0.2):
+        """
+        Wait while working widget active
+        """
+
+        working_widget = self.find('.working-widget', By.CSS_SELECTOR, widget)
+
+        self.wait(implicit)
+        WebDriverWait(self.driver, self.request_timeout).until_not(
+            lambda d: working_widget.is_displayed()
+        )
+        self.wait(0.5)
+
     def xpath_has_val(self, attr, val):
         """
         Create xpath expression for matching a presence of item in attribute
@@ -687,9 +749,16 @@ class UI_driver(object):
 
     def _button_click(self, selector, parent, name=''):
         btn = self.find(selector, By.CSS_SELECTOR, parent, strict=True)
+
+        # The small timeout (up to 5 seconds) allows to prevent exceptions when
+        # driver attempts to click a button before it is rendered.
+        WebDriverWait(self.driver, 5, 0.2).until(
+            lambda d: btn.is_displayed(),
+            'Button is not displayed: %s' % (name or selector)
+        )
         self.move_to_element_in_page(btn)
+
         disabled = btn.get_attribute("disabled")
-        assert btn.is_displayed(), 'Button is not displayed: %s' % name
         assert not disabled, 'Invalid button state: disabled. Button: %s' % name
         btn.click()
         self.wait_for_request()
@@ -769,7 +838,8 @@ class UI_driver(object):
             parent = self.get_form()
         tb = self.find(selector, By.CSS_SELECTOR, parent, strict=True)
         try:
-            tb.clear()
+            tb.send_keys(Keys.CONTROL + 'a')
+            tb.send_keys(Keys.DELETE)
             tb.send_keys(value)
         except InvalidElementStateException as e:
             msg = "Invalid Element State, el: %s, value: %s, error: %s" % (selector, value, e)
@@ -807,6 +877,11 @@ class UI_driver(object):
                 parent = self.get_form()
         self.fill_text(search_field_s, value, parent)
 
+    def apply_search_filter(self, value):
+        self.fill_search_filter(value)
+        actions = ActionChains(self.driver)
+        actions.send_keys(Keys.ENTER).perform()
+
     def add_multivalued(self, name, value, parent=None):
         """
         Add new value to multivalued textbox
@@ -955,7 +1030,8 @@ class UI_driver(object):
         self.wait_for_request()
 
         list_cnt = self.find('.combobox-widget-list', By.CSS_SELECTOR, cb, strict=True)
-        opt_s = "select[name=list] option[value='%s']" % value
+        opt_s = 'select[name=list] option'
+        opt_s += "[value='%s']" % value if value else ':not([value])'
         option = self.find(opt_s, By.CSS_SELECTOR, cb)
 
         if combobox_input:
@@ -1138,6 +1214,7 @@ class UI_driver(object):
             val = el.text
         return val
 
+    @repeat_on_stale_parent_reference
     def has_record(self, pkey, parent=None, table_name=None):
         """
         Check if table contains specific record.
@@ -1541,7 +1618,7 @@ class UI_driver(object):
             self.delete_record(pkey, data.get('del'))
             self.close_notifications()
 
-    def add_table_record(self, name, data, parent=None):
+    def add_table_record(self, name, data, parent=None, add_another=False):
         """
         Add record to dnsrecord table, association table and similar
         """
@@ -1554,9 +1631,25 @@ class UI_driver(object):
         btn.click()
         self.wait()
         self.fill_fields(data['fields'])
-        self.dialog_button_click('add')
+
+        if not add_another:
+            self.dialog_button_click('add')
+            self.wait_for_request()
+
+    def add_another_table_record(self, data, add_another=False):
+        """
+        Add table record after creating previous one in the same dialog
+        TODO: Create class to manipulate such type of dialog
+        """
+        self.dialog_button_click('add_and_add_another')
         self.wait_for_request()
 
+        self.fill_fields(data['fields'])
+
+        if not add_another:
+            self.dialog_button_click('add')
+            self.wait_for_request()
+
     def prepare_associations(
             self, pkeys, facet=None, facet_btn='add', member_pkeys=None,
             confirm_btn='add', search=False):
@@ -1879,6 +1972,8 @@ class UI_driver(object):
 
         cmd (str): command to run
         """
+        if tasks.is_fips_enabled():
+            pytest.skip("paramiko is not compatible with FIPS mode")
 
         login = self.config.get('ipa_admin')
         hostname = self.config.get('ipa_server')
@@ -1898,6 +1993,7 @@ class UI_driver(object):
         finally:
             ssh.close()
 
+    @dismiss_unexpected_alert
     def has_class(self, el, cls):
         """
         Check if el has CSS class
@@ -1905,11 +2001,23 @@ class UI_driver(object):
         class_attr = el.get_attribute("class")
         return bool(class_attr) and cls in class_attr.split()
 
+    def has_form_error(self, name):
+        """
+        Check if form field has error
+        TODO: Move to some mixin class
+        """
+        form_group = self.find(
+            '//input[@name="{}"]/ancestor'
+            '::div[contains(@class, "form-group")]'.format(name),
+            By.XPATH
+        )
+        return self.has_class(form_group, 'has-error')
+
     def skip(self, reason):
         """
         Skip tests
         """
-        raise unittest.SkipTest(reason)
+        pytest.skip(reason)
 
     def assert_text(self, selector, value, parent=None):
         """
