@@ -82,19 +82,19 @@ def update_idfn(update):
 
 
 @pytest.fixture(scope='class')
-def certmap_rule(request):
+def certmap_rule(request, xmlrpc_setup):
     tracker = CertmapruleTracker(**certmaprule_create_params)
     return tracker.make_fixture(request)
 
 
 @pytest.fixture(scope='class')
-def certmap_rule_trusted_domain(request):
+def certmap_rule_trusted_domain(request, xmlrpc_setup):
     tracker = CertmapruleTracker(**certmaprule_create_trusted_params)
     return tracker.make_fixture(request)
 
 
 @pytest.fixture(scope='class')
-def certmap_config(request):
+def certmap_config(request, xmlrpc_setup):
     tracker = CertmapconfigTracker()
     return tracker.make_fixture(request)
 
@@ -206,7 +206,7 @@ def certmap_user(request):
 
 def addcertmap_id(options):
     if options:
-        return u', '.join([k for k in options])
+        return u', '.join(list(options))
     else:
         return u' '
 
@@ -326,7 +326,7 @@ def change_permissions_bindtype(perm, bindtype):
 
 
 @pytest.fixture(scope='class')
-def bindtype_permission(request):
+def bindtype_permission(request, xmlrpc_setup):
     orig_bindtype = {}
     # set bindtype to permission to actually test the permission
     for perm_name in certmaprule_permissions.values():
@@ -451,7 +451,7 @@ class TestPermission(XMLRPC_test):
         ) as ewe:
             find = certmap_rule.make_find_command()
             res = find(**{k: v for k, v in certmaprule_create_params.items()
-                          if k is not u'dn'})
+                          if k != u'dn'})
             ewe.send(res)
 
     def test_update(self, certmap_rule, certmap_user_permissions):
