@@ -19,13 +19,12 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
-Test the `ipalib/plugins/role.py` module.
+Test the `ipaserver/plugins/role.py` module.
 """
 
 from ipalib import api, errors
 from ipatests.test_xmlrpc import objectclasses
-from ipatests.test_xmlrpc.xmlrpc_test import (Declarative, fuzzy_digits,
-                                              fuzzy_uuid)
+from ipatests.test_xmlrpc.xmlrpc_test import Declarative, fuzzy_uuid
 from ipapython.dn import DN
 import pytest
 
@@ -200,7 +199,6 @@ class test_role(Declarative):
                     'cn': [role1],
                     'description': [u'role desc 1'],
                     'memberof_privilege': [privilege1],
-                    'objectclass': objectclasses.role,
                 }
             ),
         ),
@@ -222,7 +220,6 @@ class test_role(Declarative):
                     'cn': [role1],
                     'description': [u'role desc 1'],
                     'memberof_privilege': [privilege1],
-                    'objectclass': objectclasses.role,
                 }
             ),
         ),
@@ -244,7 +241,6 @@ class test_role(Declarative):
                     'cn': [role1],
                     'description': [u'role desc 1'],
                     'memberof_privilege': [privilege1],
-                    'objectclass': objectclasses.role,
                 }
             ),
         ),
@@ -292,9 +288,47 @@ class test_role(Declarative):
         ),
 
 
+        dict(
+            desc='Search for %r with members' % role1,
+            command=('role_find', [role1], {'no_members': False}),
+            expected=dict(
+                count=1,
+                truncated=False,
+                summary=u'1 role matched',
+                result=[
+                    {
+                        'dn': role1_dn,
+                        'cn': [role1],
+                        'description': [u'role desc 1'],
+                        'member_group': [group1],
+                        'memberof_privilege': [privilege1],
+                    },
+                ],
+            ),
+        ),
+
+
         dict(
             desc='Search for %r' % role1,
             command=('role_find', [role1], {}),
+            expected=dict(
+                count=1,
+                truncated=False,
+                summary=u'1 role matched',
+                result=[
+                    {
+                        'dn': role1_dn,
+                        'cn': [role1],
+                        'description': [u'role desc 1'],
+                    },
+                ],
+            ),
+        ),
+
+
+        dict(
+            desc='Search for %r with members' % search,
+            command=('role_find', [search], {'no_members': False}),
             expected=dict(
                 count=1,
                 truncated=False,
@@ -324,8 +358,6 @@ class test_role(Declarative):
                         'dn': role1_dn,
                         'cn': [role1],
                         'description': [u'role desc 1'],
-                        'member_group': [group1],
-                        'memberof_privilege': [privilege1],
                     },
                 ],
             ),
@@ -351,8 +383,8 @@ class test_role(Declarative):
 
 
         dict(
-            desc='Search for %r' % role1,
-            command=('role_find', [role1], {}),
+            desc='Search for %r with members' % role1,
+            command=('role_find', [role1], {'no_members': False}),
             expected=dict(
                 count=1,
                 truncated=False,
@@ -371,8 +403,26 @@ class test_role(Declarative):
 
 
         dict(
-            desc='Search for %r' % search,
-            command=('role_find', [search], {}),
+            desc='Search for %r' % role1,
+            command=('role_find', [role1], {}),
+            expected=dict(
+                count=1,
+                truncated=False,
+                summary=u'1 role matched',
+                result=[
+                    {
+                        'dn': role1_dn,
+                        'cn': [role1],
+                        'description': [u'role desc 1'],
+                    },
+                ],
+            ),
+        ),
+
+
+        dict(
+            desc='Search for %r with members' % search,
+            command=('role_find', [search], {'no_members': False}),
             expected=dict(
                 count=2,
                 truncated=False,
@@ -395,6 +445,29 @@ class test_role(Declarative):
         ),
 
 
+        dict(
+            desc='Search for %r' % search,
+            command=('role_find', [search], {}),
+            expected=dict(
+                count=2,
+                truncated=False,
+                summary=u'2 roles matched',
+                result=[
+                    {
+                        'dn': role1_dn,
+                        'cn': [role1],
+                        'description': [u'role desc 1'],
+                    },
+                    {
+                        'dn': role2_dn,
+                        'cn': [role2],
+                        'description': [u'role desc 2'],
+                    },
+                ],
+            ),
+        ),
+
+
         dict(
             desc='Update %r' % role1,
             command=(
@@ -527,7 +600,6 @@ class test_role(Declarative):
                     'dn': role1_dn,
                     'cn': [role1],
                     'description': [u'New desc 1'],
-                    'objectclass': objectclasses.role,
                 }
             ),
         ),
@@ -549,7 +621,6 @@ class test_role(Declarative):
                     'dn': role1_dn,
                     'cn': [role1],
                     'description': [u'New desc 1'],
-                    'objectclass': objectclasses.role,
                 }
             ),
         ),
