@@ -11,10 +11,13 @@ from ipatests.util import assert_deepequal, get_group_dn
 
 class GroupTracker(Tracker):
     """ Class for host plugin like tests """
-    retrieve_keys = {u'dn', u'cn', u'gidnumber', u'member_user',
-                     u'member_group', u'description',
-                     u'memberof_group', u'memberofindirect_group',
-                     u'memberindirect_group', u'memberindirect_user'}
+    retrieve_keys = {
+        'dn', 'cn', 'gidnumber', 'member_user', 'member_group',
+        'member_service', 'description', 'memberof_group',
+        'memberofindirect_group', 'memberindirect_group',
+        'memberindirect_user', 'memberindirect_service',
+        'membermanager_group', 'membermanager_user'
+    }
 
     retrieve_all_keys = retrieve_keys | {u'ipauniqueid', u'objectclass'}
 
@@ -30,7 +33,7 @@ class GroupTracker(Tracker):
         self.dn = get_group_dn(self.cn)
 
     def make_create_command(self, nonposix=False, external=False,
-                            force=True, *args, **kwargs):
+                            *args, **kwargs):
         """ Make function that creates a group using 'group-add' """
         return self.make_command('group_add', self.cn,
                                  description=self.description,
@@ -68,6 +71,16 @@ class GroupTracker(Tracker):
         self.exists = True
         return self.make_command('group_detach', self.cn)
 
+    def make_add_member_manager_command(self, options={}):
+        return self.make_command(
+            'group_add_member_manager', self.cn, **options
+        )
+
+    def make_remove_member_manager_command(self, options={}):
+        return self.make_command(
+            'group_remove_member_manager', self.cn, **options
+        )
+
     def track_create(self):
         """ Updates expected state for group creation"""
         self.attrs = dict(
@@ -112,19 +125,25 @@ class GroupTracker(Tracker):
         )
 
     def add_member(self, options):
-        """ Add a member (group OR user) and performs check """
+        """ Add a member (group OR user OR service) and performs check """
         if u'user' in options:
             try:
                 self.attrs[u'member_user'] =\
                     self.attrs[u'member_user'] + [options[u'user']]
-            except KeyError as ex:
+            except KeyError:
                 self.attrs[u'member_user'] = [options[u'user']]
         elif u'group' in options:
             try:
                 self.attrs[u'member_group'] =\
                     self.attrs[u'member_group'] + [options[u'group']]
-            except KeyError as ex:
+            except KeyError:
                 self.attrs[u'member_group'] = [options[u'group']]
+        elif u'service' in options:
+            try:
+                self.attrs[u'member_service'] =\
+                    self.attrs[u'member_service'] + [options[u'service']]
+            except KeyError:
+                self.attrs[u'member_service'] = [options[u'service']]
 
         command = self.make_add_member_command(options)
         result = command()
@@ -136,22 +155,63 @@ class GroupTracker(Tracker):
             self.attrs[u'member_user'].remove(options[u'user'])
         elif u'group' in options:
             self.attrs[u'member_group'].remove(options[u'group'])
+        elif u'service' in options:
+            self.attrs[u'member_service'].remove(options[u'service'])
 
         try:
             if not self.attrs[u'member_user']:
                 del self.attrs[u'member_user']
-        except KeyError as ex:
+        except KeyError:
             pass
         try:
             if not self.attrs[u'member_group']:
                 del self.attrs[u'member_group']
-        except KeyError as ex:
+        except KeyError:
+            pass
+        try:
+            if not self.attrs[u'member_service']:
+                del self.attrs[u'member_service']
+        except KeyError:
             pass
 
         command = self.make_remove_member_command(options)
         result = command()
         self.check_remove_member(result)
 
+    def add_member_manager(self, options):
+        """Add a member manager (user or group) and perform checks"""
+        if "user" in options:
+            members = self.attrs.setdefault("membermanager_user", [])
+            members.append(options["user"])
+        elif "group" in options:
+            members = self.attrs.setdefault("membermanager_group", [])
+            members.append(options["group"])
+
+        command = self.make_add_member_manager_command(options)
+        result = command()
+        self.check_add_member_manager(result)
+
+    def remove_member_manager(self, options):
+        if "user" in options:
+            members = self.attrs["membermanager_user"]
+            members.remove(options["user"])
+            if not members:
+                self.attrs.pop("membermanager_user")
+        elif "group" in options:
+            members = self.attrs["membermanager_group"]
+            members.remove(options["group"])
+            if not members:
+                self.attrs.pop("membermanager_group")
+
+        command = self.make_remove_member_manager_command(options)
+        result = command()
+        self.check_remove_member_manager(result)
+
+    def retrieve(self, all=False, raw=False):
+        command = self.make_retrieve_command(all=all, raw=raw)
+        result = command()
+        self.check_retrieve(result, all=all, raw=raw)
+
     def check_create(self, result):
         """ Checks 'group_add' command result """
         assert_deepequal(dict(
@@ -207,7 +267,7 @@ class GroupTracker(Tracker):
         """ Checks 'group_add_member' command result """
         assert_deepequal(dict(
             completed=1,
-            failed={u'member': {u'group': (), u'user': ()}},
+            failed={u'member': {u'group': (), u'user': (), u'service': ()}},
             result=self.filter_attrs(self.add_member_keys)
         ), result)
 
@@ -216,7 +276,7 @@ class GroupTracker(Tracker):
         when expected result is failure of the operation"""
         expected = dict(
             completed=0,
-            failed={u'member': {u'group': (), u'user': ()}},
+            failed={u'member': {u'group': (), u'user': (), u'service': ()}},
             result=self.filter_attrs(self.add_member_keys)
         )
         if not options:
@@ -230,6 +290,9 @@ class GroupTracker(Tracker):
         elif u'group' in options:
             expected[u'failed'][u'member'][u'group'] = [(
                 options[u'group'], u'no such entry')]
+        elif u'service' in options:
+            expected[u'failed'][u'member'][u'service'] = [(
+                options[u'service'], u'no such entry')]
 
         assert_deepequal(expected, result)
 
@@ -238,7 +301,7 @@ class GroupTracker(Tracker):
         when expected result is failure of the operation"""
         expected = dict(
             completed=0,
-            failed={u'member': {u'group': (), u'user': ()}},
+            failed={u'member': {u'group': (), u'user': (), u'service': ()}},
             result=self.filter_attrs(self.add_member_keys)
         )
         if u'user' in options:
@@ -247,6 +310,9 @@ class GroupTracker(Tracker):
         elif u'group' in options:
             expected[u'failed'][u'member'][u'group'] = [(
                 options[u'group'], u'This entry is not a member')]
+        elif u'service' in options:
+            expected[u'failed'][u'member'][u'service'] = [(
+                options[u'service'], u'This entry is not a member')]
 
         assert_deepequal(expected, result)
 
@@ -254,6 +320,16 @@ class GroupTracker(Tracker):
         """ Checks 'group_remove_member' command result """
         self.check_add_member(result)
 
+    def check_add_member_manager(self, result):
+        assert_deepequal(dict(
+            completed=1,
+            failed={'membermanager': {'group': (), 'user': ()}},
+            result=self.filter_attrs(self.add_member_keys)
+        ), result)
+
+    def check_remove_member_manager(self, result):
+        self.check_add_member_manager(result)
+
     def check_detach(self, result):
         """ Checks 'group_detach' command result """
         assert_deepequal(dict(
