@@ -35,19 +35,14 @@ from contextlib import contextmanager
 from pprint import pformat
 
 import six
-import ldap
-import ldap.sasl
-import ldap.modlist
 
 import ipalib
 from ipalib import api
 from ipalib.plugable import Plugin
 from ipalib.request import context
 from ipapython.dn import DN
-from ipapython.ipaldap import ldap_initialize
 from ipapython.ipautil import run
 
-
 try:
     # not available with client-only wheel packages
     from ipalib.install.kinit import kinit_keytab, kinit_password
@@ -60,6 +55,15 @@ try:
 except ImportError:
     paths = None
 
+try:
+    # not available with optional python-ldap
+    import ldap
+except ImportError:
+    pass
+else:
+    import ldap.sasl
+    import ldap.modlist
+    from ipapython.ipaldap import ldap_initialize
 
 if six.PY3:
     unicode = str
@@ -67,12 +71,16 @@ if six.PY3:
 
 PYTEST_VERSION = tuple(int(v) for v in pytest.__version__.split('.'))
 
+# settings are configured by conftest
+IPACLIENT_UNITTESTS = None
+SKIP_IPAAPI = None
+PRETTY_PRINT = None
+
 
 def check_ipaclient_unittests(reason="Skip in ipaclient unittest mode"):
     """Call this in a package to skip the package in ipaclient-unittest mode
     """
-    config = pytest.config  # pylint: disable=no-member
-    if config.getoption('ipaclient_unittests', False):
+    if IPACLIENT_UNITTESTS:
         if PYTEST_VERSION[0] >= 3:
             # pytest 3+ does no longer allow pytest.skip() on module level
             # pylint: disable=unexpected-keyword-arg
@@ -85,8 +93,7 @@ def check_ipaclient_unittests(reason="Skip in ipaclient unittest mode"):
 def check_no_ipaapi(reason="Skip tests that needs an IPA API"):
     """Call this in a package to skip the package in no-ipaapi mode
     """
-    config = pytest.config  # pylint: disable=no-member
-    if config.getoption('skip_ipaapi', False):
+    if SKIP_IPAAPI:
         if PYTEST_VERSION[0] >= 3:
             # pylint: disable=unexpected-keyword-arg
             raise pytest.skip.Exception(reason, allow_module_level=True)
@@ -95,14 +102,15 @@ def check_no_ipaapi(reason="Skip tests that needs an IPA API"):
             raise pytest.skip(reason)
 
 
-class TempDir(object):
+class TempDir:
     def __init__(self):
         self.__path = tempfile.mkdtemp(prefix='ipa.tests.')
         assert self.path == self.__path
 
     def __get_path(self):
         assert path.abspath(self.__path) == self.__path
-        assert self.__path.startswith('/tmp/ipa.tests.')
+        assert self.__path.startswith(path.join(tempfile.gettempdir(),
+                                                'ipa.tests.'))
         assert path.isdir(self.__path) and not path.islink(self.__path)
         return self.__path
     path = property(__get_path)
@@ -185,8 +193,8 @@ def assert_not_equal(val1, val2):
     assert val1 != val2, '%r == %r' % (val1, val2)
 
 
-class Fuzzy(object):
-    """
+class Fuzzy:
+    r"""
     Perform a fuzzy (non-strict) equality tests.
 
     `Fuzzy` instances will likely be used when comparing nesting
@@ -212,7 +220,7 @@ class Fuzzy(object):
     Use of a regular expression by default implies the ``unicode`` type, so
     comparing with an ``str`` instance will evaluate to ``False``:
 
-    >>> phone.type is six.text_type
+    >>> phone.type is str
     True
     >>> b'123-456-7890' == phone
     False
@@ -280,7 +288,7 @@ class Fuzzy(object):
         :param test: A callable used to perform equality test, e.g.
             ``lambda other: other >= 18``
         """
-        assert regex is None or isinstance(regex, six.string_types)
+        assert regex is None or isinstance(regex, str)
         assert test is None or callable(test)
         if regex is None:
             self.re = None
@@ -288,7 +296,7 @@ class Fuzzy(object):
             self.re = re.compile(regex)
             if type is None:
                 type = unicode
-            assert type in (unicode, bytes, six.string_types)
+            assert type in (unicode, bytes, str)
         self.regex = regex
         self.type = type
         self.test = test
@@ -383,12 +391,7 @@ def assert_deepequal(expected, got, doc='', stack=tuple()):
     Note that lists and tuples are considered equivalent, and the order of
     their elements does not matter.
     """
-    try:
-        pretty_print = pytest.config.getoption("pretty_print")
-    except (AttributeError, ValueError):
-        pretty_print = False
-
-    if pretty_print:
+    if PRETTY_PRINT:
         expected_str = struct_to_string(expected, EXPECTED_LEN)
         got_str = struct_to_string(got, GOT_LEN)
     else:
@@ -400,7 +403,7 @@ def assert_deepequal(expected, got, doc='', stack=tuple()):
     if isinstance(got, tuple):
         got = list(got)
     if isinstance(expected, DN):
-        if isinstance(got, six.string_types):
+        if isinstance(got, str):
             got = DN(got)
     if (
         not (isinstance(expected, Fuzzy)
@@ -523,7 +526,7 @@ def is_prop(prop):
     return type(prop) is property
 
 
-class ClassChecker(object):
+class ClassChecker:
     __cls = None
     __subcls = None
 
@@ -547,11 +550,11 @@ class ClassChecker(object):
             'get_subcls()'
         )
 
-    def teardown(self):
-        """
-        nose tear-down fixture.
-        """
-        context.__dict__.clear()
+    @pytest.fixture(autouse=True)
+    def classchecker_setup(self, request):
+        def fin():
+            context.__dict__.clear()
+        request.addfinalizer(fin)
 
 
 def get_api(**kw):
@@ -584,7 +587,7 @@ def create_test_api(**kw):
     return (api, home)
 
 
-class PluginTester(object):
+class PluginTester:
     __plugin = None
 
     def __get_plugin(self):
@@ -619,14 +622,14 @@ class PluginTester(object):
         o = api[namespace][self.plugin.__name__]
         return (o, api, home)
 
-    def teardown(self):
-        """
-        nose tear-down fixture.
-        """
-        context.__dict__.clear()
+    @pytest.fixture(autouse=True)
+    def plugintester_setup(self, request):
+        def fin():
+            context.__dict__.clear()
+        request.addfinalizer(fin)
 
 
-class dummy_ugettext(object):
+class dummy_ugettext:
     __called = False
 
     def __init__(self, translation=None):
@@ -655,7 +658,7 @@ class dummy_ugettext(object):
         self.__called = False
 
 
-class dummy_ungettext(object):
+class dummy_ungettext:
     __called = False
 
     def __init__(self):
@@ -676,7 +679,7 @@ class dummy_ungettext(object):
         return self.translation_plural
 
 
-class DummyMethod(object):
+class DummyMethod:
     def __init__(self, callback, name):
         self.__callback = callback
         self.__name = name
@@ -685,7 +688,7 @@ class DummyMethod(object):
         return self.__callback(self.__name, args, kw)
 
 
-class DummyClass(object):
+class DummyClass:
     def __init__(self, *calls):
         self.__calls = calls
         self.__i = 0
@@ -729,7 +732,7 @@ class DummyClass(object):
         return self.__i == len(self.__calls)
 
 
-class MockLDAP(object):
+class MockLDAP:
     def __init__(self):
         self.connection = ldap_initialize(
             'ldap://{host}'.format(host=ipalib.api.env.host)
